/*
// main.cc
//
// Andy, ab012f@uhura.cc.rochester.edu, 15 Dec 1998
// Time-stamp: <1998-12-16 07:19:32 ender>
*/

/* Copyright (C) 1998 Andrew Burke <ab012f@uhura.cc.rochester.edu>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#include <libicq.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>

int logged_in = 0; //this is crappy... ?

int Error(char *fmt, ...) {
  char err_buf[512]; //erm, buffer overflow? not suid, so... ?
  int i;

  va_list args;
  va_start(args, fmt);
  i = vsprintf(err_buf, fmt, args);
  va_end(args);

  fprintf(stderr, err_buf);
  
  return i;
}

static void Login(void *data) {
  ICQ_Change_Status(STATUS_ONLINE);
  logged_in = 1;
}

int main(int argc, char *argv[]) {

  int i = 0, len = 0, debug = 0;
  long uin = 0;
  char *msg;
  
  if(argc < 2) {
    Error("Usage: %s <uin/nick> <msg text> <options>\n", argv[0]);
    Error(" --help for help\n");
    Error(" --debug to turn on debugging... sorta\n");
    Error(" --daemon !!unimplemented!!\n");
    exit(0);
  }

  for(i = 1; i<argc; i++) { // go through args...
    if(!strcmp(argv[i], "--help")) {
      Error("\nUsage: %s <uin/nick> <msg text>\n", argv[0]);
      Error("  <uin/nick> should be the target user's UIN/Nickname\n");
      Error("  <msg text> can be a max of 256 chars\n");
      Error("  (nicknames must be in your ~/.icq/contacts.rc file)\n");
      Error("\n");
      Error("example:\n\n");
      Error("  %s 1109332 hey andy, this is a stupid program...\n\n", argv[0]);
      Error("use quotes for special characters:\n\n");
      Error("  %s ender \"hey, here's a msg with special chars in it...\"\n\n", argv[0]);
      exit(0);
    }
    if(!strcmp(argv[i], "--debug")) debug = 1;
    if(!strcmp(argv[i], "--daemon")) {
      Error("Daemon functionality unimplemented...\n");
      Error("Send me your diffs... :>\n");
      exit(0);
    }
  }

  if(!(msg = (char *)malloc(256))) {
    Error("out of memory... you lack ram, buy more");
    exit(0);
  }
  msg[0] = '\0';

  ICQ_Register_Callback(EVENT_LOGIN, Login);

  if(!ICQ_Connect()) {
    Error("Could not connect...\n");
    exit(0);
  }

  for(i=2; i<argc; i++) {

    if(!strcmp(argv[i], "--debug")) continue; //crappy hack...
    if(!strcmp(argv[i], "--help")) continue;

    len += strlen(argv[i]);
    if(len < 256) strcat(msg, argv[i]);
    else break;

    len++;
    if(len < 256) strcat(msg, " ");
    else break;

  }

  if(len >= 256) {
    Error("Buffer Exceeded... message truncated to:\n");
    Error("%s\n", msg);
  }

  if(!isdigit(argv[1][0])) {
    for(i=0; i<Num_Contacts; i++) {
      if(debug) {
	Error("argv[1] = <%s> / Contacts[%d].nick = <%s>\n", argv[1], i, Contacts[i].nick);
      }
      if(!strcmp(argv[1], Contacts[i].nick)) {
	uin = Contacts[i].uin;
	break;
      }
    }
    if(uin == 0) {
      Error("<%s> not found in contacts.rc\n", argv[1]);
      exit(0);
    }
  } else uin = atol(argv[1]);

  while(1) {
    if(debug) printf(".");
    ICQ_Check_Response(10000);
    if(logged_in) break;
  }

  if(debug) Error("uin: <%d>\n", uin);
  if(debug) Error("msg: <%s>\n", msg);
  ICQ_Send_Message(uin, msg);
  ICQ_Check_Response(10000);

  if(debug) Error("message sent...\n");
  ICQ_Change_Status(STATUS_OFFLINE);
  ICQ_Disconnect();
  
}
