/* Copyright (C) 1998 Sean Gabriel <gabriel@montana.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifdef HAVE_GNOME
#include <gnome.h>
#else
#include <gtk/gtk.h>
#endif

#include <stdlib.h>
#include <libicq.h>
#include <config.h>
#include <libintl.h>

#include "event.h"
#include "contactlist.h"
#include "labeledentry.h"

#include "pixmaps/green_flower.xpm"
#include "pixmaps/red_flower.xpm"
#include "pixmaps/away_flower.xpm"
#include "pixmaps/na_flower.xpm"
#include "pixmaps/occ_flower.xpm"
#include "pixmaps/dnd_flower.xpm"

#include "pixmaps/message.xpm"
#include "pixmaps/message_gray.xpm"
#include "pixmaps/message_away.xpm"
#include "pixmaps/message_na.xpm"
#include "pixmaps/message_dnd.xpm"
#include "pixmaps/message_occ.xpm"


GdkPixmap* message_pixmap, *gray_message_pixmap, *away_message_pixmap,
	*na_message_pixmap, *dnd_message_pixmap, *occ_message_pixmap;
GdkBitmap* mask;
GtkWidget* contact_list;

#ifdef HAVE_GNOME

static void about_cb(GtkWidget *widget, gpointer data)
{
    GtkWidget* about;
    
    gchar* authors[] = {
            "Sean Gabriel <gabriel@montana.com>",
            "Andrew Burke <ab012f@uhura.cc.rochester.edu>",
            NULL
    };

    about = gnome_about_new(_("gicq"), VERSION,
                             _("(C) 1998 Sean Gabriel"),
                             authors,
                             "ICQ protocol back-end:\n             Matt Smith <mds1281@rit.edu>\n \nAn ICQ-compatible instant messaging client",
                             NULL);
                             
    gtk_widget_show(about);
}


static void ShowOnline(GtkWidget *widget, gpointer data)
{
	static int offlist_vis;
		
	if(GTK_WIDGET_VISIBLE(CONTACT_LIST(contact_list)->online_toggle))
	{
		offlist_vis = GTK_WIDGET_VISIBLE(CONTACT_LIST(contact_list)->offline_list);
		gtk_widget_hide(CONTACT_LIST(contact_list)->online_toggle);
		gtk_widget_hide(CONTACT_LIST(contact_list)->offline_toggle);
		gtk_widget_hide(CONTACT_LIST(contact_list)->offline_list);
	}
	else
	{
		gtk_widget_show(CONTACT_LIST(contact_list)->online_toggle);
		gtk_widget_show(CONTACT_LIST(contact_list)->offline_toggle);
		if(offlist_vis) gtk_widget_show(CONTACT_LIST(contact_list)->offline_list);
	}
}


GnomeUIInfo helpmenu[] = {
	{GNOME_APP_UI_ITEM, N_("About"), NULL, about_cb, NULL, NULL, 
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL},
/*
	GNOMEUIINFO_SEPARATOR,

	GNOMEUIINFO_HELP("help-browser"),
*/
	GNOMEUIINFO_END
};


GnomeUIInfo menumenu[] = {
	{GNOME_APP_UI_ITEM, N_("Search/Add Users"), NULL, StartSearch, NULL, NULL, 
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH, 0, 0, NULL},

	{GNOME_APP_UI_TOGGLEITEM, N_("Show Only Online Users"), NULL, ShowOnline, NULL, NULL, 
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
/*	
	{GNOME_APP_UI_ITEM, N_("Preferences"), NULL, NULL, NULL, NULL, 
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL},
*/	
	GNOMEUIINFO_SEPARATOR,
	
	{GNOME_APP_UI_ITEM, N_("Exit"), NULL, Shutdown, NULL, NULL, 
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 0, 0, NULL},

	GNOMEUIINFO_END
};

GnomeUIInfo mainmenu[] = {
	GNOMEUIINFO_SUBTREE(N_("Menu"), menumenu),
	GNOMEUIINFO_SUBTREE(N_("Help"), helpmenu),
	GNOMEUIINFO_END
};

#else

void About(GtkWidget* widget, gpointer data)
{
	GtkWidget* window;
	GtkWidget* vbox;
	GtkWidget* label;
	GtkWidget* close;
	
	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "About");
	
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(vbox), 5);

	label = gtk_label_new("GTK Interface:\n\
		Sean Gabriel <gabriel@montana.com>\n\
		Andrew Burke <ab012f@uhura.cc.rochester.edu>\n\n\
		ICQ Protocol back-end:\n\
		Matt Smith <mds1281@rit.edu>");
	
	close = gtk_button_new_with_label("Close");
	
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 5);
	gtk_box_pack_start(GTK_BOX(vbox), close, FALSE, FALSE, 5);

	gtk_signal_connect_object(GTK_OBJECT(close), "clicked",
		GTK_SIGNAL_FUNC(gtk_widget_destroy), (gpointer)window);

	gtk_widget_show(label);
	gtk_widget_show(close);
	gtk_widget_show(vbox);
	gtk_widget_show(window);
}


GtkWidget* MakeMenubar()
{
	GtkWidget* menubar, *menubar_menu, *menubar_help;
	GtkWidget* menu, *menu_search_item, *menu_prefs_item, *menu_all_item, *menu_quit_item,
		*help, *help_about_item;

	menubar = gtk_menu_bar_new();
	gtk_container_disable_resize(GTK_CONTAINER(menubar));
	menubar_menu = gtk_menu_item_new_with_label("Menu");
	menubar_help = gtk_menu_item_new_with_label("Help");
	menu = gtk_menu_new();
	help = gtk_menu_new();
	menu_search_item = gtk_menu_item_new_with_label("Search/Add Users");
/*	menu_prefs_item = gtk_menu_item_new_with_label("Preferences");
	menu_all_item = gtk_menu_item_new_with_label("Show All"); */
	menu_quit_item = gtk_menu_item_new_with_label("Quit");
	help_about_item = gtk_menu_item_new_with_label("About");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menubar_menu), menu);
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menubar_help), help);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menubar_menu);
	gtk_menu_bar_append(GTK_MENU_BAR(menubar), menubar_help);
	gtk_menu_item_right_justify(GTK_MENU_ITEM(menubar_help));
	gtk_menu_append(GTK_MENU(menu), menu_search_item);
/*	gtk_menu_append(GTK_MENU(menu), menu_prefs_item);
	gtk_menu_append(GTK_MENU(menu), menu_all_item); */
	gtk_menu_append(GTK_MENU(menu), menu_quit_item);
	gtk_menu_append(GTK_MENU(help), help_about_item);
	gtk_widget_show(menu_search_item);
/*	gtk_widget_show(menu_prefs_item);
	gtk_widget_show(menu_all_item); */
	gtk_widget_show(menu_quit_item);
	gtk_widget_show(help_about_item);
	gtk_widget_show(menubar_menu);
	gtk_widget_show(menubar_help);

	gtk_signal_connect_object(GTK_OBJECT(menu_search_item), "activate",
		GTK_SIGNAL_FUNC(StartSearch), NULL);

	gtk_signal_connect(GTK_OBJECT(menu_quit_item), "activate", 
		GTK_SIGNAL_FUNC(Shutdown), NULL);
	
	gtk_signal_connect(GTK_OBJECT(help_about_item), "activate", 
		GTK_SIGNAL_FUNC(About), NULL);

	return menubar;
}

#endif

GtkWidget* MakeStatusButton()
{
	GtkWidget* status_button;
	GtkWidget* status_menu, *status_menu_item, *status[6];
	gchar* status_label[6];
	GtkWidget* hbox, *label, *pixmap;
	GdkPixmap* gdk_pixmap;
	GdkBitmap* mask;
	GtkStyle* style;
	char** status_xpm[6];
	int x;

	status_xpm[0] = green_flower_xpm;
	status_xpm[1] = away_flower_xpm;
	status_xpm[2] = na_flower_xpm;
	status_xpm[3] = occ_flower_xpm;
	status_xpm[4] = dnd_flower_xpm;
	status_xpm[5] = red_flower_xpm;

	status_label[0] = "Online";
	status_label[1] = "Away";
	status_label[2] = "Not Available";
	status_label[3] = "Occupied";
	status_label[4] = "Do Not Disturb";
	status_label[5] = "Offline";

	status_menu = gtk_menu_new();
	style = gtk_widget_get_style(status_menu);

	gtk_widget_realize(status_menu);

	for(x = 0; x < 6; x++)
	{
		status_menu_item = gtk_menu_item_new();
		gdk_pixmap = gdk_pixmap_create_from_xpm_d(status_menu->window, &mask,
			&style->bg[GTK_STATE_NORMAL], (gchar **)status_xpm[x]);
		hbox = gtk_hbox_new(FALSE, 6);
		pixmap = gtk_pixmap_new(gdk_pixmap, mask);
		label = gtk_label_new(status_label[x]);
		gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
		gtk_widget_show(pixmap);
		gtk_widget_show(label);
		gtk_widget_show(hbox);
		gtk_widget_show(status_menu_item);
		gtk_container_add(GTK_CONTAINER(status_menu_item), hbox);
		gtk_menu_append(GTK_MENU(status_menu), status_menu_item);
		gtk_signal_connect_object(GTK_OBJECT(status_menu_item), "activate",
			GTK_SIGNAL_FUNC(ChangeStatus), (gpointer)x);
	}

	status_button = gtk_option_menu_new();
	gtk_menu_set_active(GTK_MENU(status_menu), 5);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(status_button), status_menu);

	return status_button;
}



int main(int argc, char* argv[])
{
	GtkWidget* window;
	GtkWidget* vbox1, *vbox2, *hbox;

	GtkWidget* menubar;
	GtkWidget* status_button;
	GtkWidget* login_window;

	GtkStyle* style;

#ifdef HAVE_GNOME
	setlocale(LC_ALL, "");
	bindtextdomain(PACKAGE, LOCALEDIR);
	textdomain(PACKAGE);

	gnome_init("gicq", NULL, argc, argv, 0, NULL);
	window = gnome_app_new("gicq", "GICQ");
#else
	gtk_init(&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "GICQ");
#endif

	gtk_signal_connect(GTK_OBJECT(window), "destroy", 
		GTK_SIGNAL_FUNC(Shutdown), NULL);

	vbox1 = gtk_vbox_new(FALSE, 0);
	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(vbox2), 5);

#ifdef HAVE_GNOME
	gnome_app_create_menus(GNOME_APP(window), mainmenu);
	gtk_menu_item_right_justify(GTK_MENU_ITEM(mainmenu[1].widget));
#else
	menubar = MakeMenubar();
#endif
	contact_list = contact_list_new();
	status_button = MakeStatusButton();

#ifdef HAVE_GNOME
	gnome_app_set_contents(GNOME_APP(window), vbox1);
#else
	gtk_container_add(GTK_CONTAINER(window), vbox1);
	gtk_box_pack_start(GTK_BOX(vbox1), menubar, FALSE, FALSE, 0);
#endif

	gtk_container_add(GTK_CONTAINER(vbox1), vbox2);
	gtk_box_pack_start(GTK_BOX(vbox2), contact_list, TRUE, TRUE, 3);
	gtk_box_pack_start(GTK_BOX(vbox2), status_button, FALSE, FALSE, 3);

	gtk_widget_realize(GTK_WIDGET(contact_list));

	style = gtk_widget_get_style(contact_list);
	gray_message_pixmap = gdk_pixmap_create_from_xpm_d(contact_list->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)message_gray_xpm);
	message_pixmap = gdk_pixmap_create_from_xpm_d(contact_list->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)message_xpm);
	away_message_pixmap = gdk_pixmap_create_from_xpm_d(contact_list->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)message_away_xpm);
	na_message_pixmap = gdk_pixmap_create_from_xpm_d(contact_list->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)message_na_xpm);
	dnd_message_pixmap = gdk_pixmap_create_from_xpm_d(contact_list->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)message_dnd_xpm);
	occ_message_pixmap = gdk_pixmap_create_from_xpm_d(contact_list->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)message_occ_xpm);

	gtk_clist_set_pixmap(GTK_CLIST(CONTACT_LIST(contact_list)->system), 0, 0,
				gray_message_pixmap, mask);

#ifndef HAVE_GNOME
	gtk_widget_show(menubar);
#endif
	gtk_widget_show(contact_list);
	gtk_widget_show(status_button);

	gtk_widget_show(vbox1);
	gtk_widget_show(vbox2);

	/* finally we want to see the window, dont we? ;) */
	gtk_widget_show(window);
	
	RegisterCallbacks();
	
	/* fire up the main event loop of gtk */
	gtk_main();
         
	/* we get here after gtk_main_quit() has been called which
	 * happens if the main window gets destroyed */
	return 0;
}










































