/* Copyright (C) 1998 Sean Gabriel <gabriel@montana.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */


#ifdef HAVE_GNOME
#include <gnome.h>
#else
#include <gtk/gtk.h>
#endif

#include <stdlib.h>
#include <libicq.h>
#include <config.h>

#include "event.h"
#include "login.h"
#include "contactlist.h"
#include "labeledentry.h"
#include "message.h"
#include "url.h"
#include "search.h"
#include "userinfo.h"


gint timeout_id;
gint select_id;
gint connection = OFFLINE;
char config_file[100];

extern GtkWidget* contact_list;

USER_INFO user_info;
USER_EXT_INFO user_ext_info;

unsigned long pending_search;

extern GtkWidget* contact_list;


static gint SelectServer(gpointer data)
{
	if(connection != OFFLINE) ICQ_Check_Response(10000);
	return TRUE;
}


static gint KeepAlive(gpointer data)
{
	if(connection == ONLINE) ICQ_Keep_Alive();
	return TRUE;
}


static void EventLogin(void* data)
{
	int i;
	
	connection = ONLINE;
	pending_search = 0;

	for(i = 0; i < Num_Contacts; i++)
		if(Contacts[i].uin > 0)
			contact_list_update_user(CONTACT_LIST(contact_list),
				Contacts[i].uin, Contacts[i].nick, Contacts[i].status);
}


static void EventDisconnect(void* data)
{
	if(connection == CONNECTING) ChangeStatus(0);
}


static void EventMessage(void* data)
{
	GtkWidget* msgbox;
	CLIENT_MESSAGE_PTR c_mesg;
	char nick[20], uin[20];

	c_mesg = (CLIENT_MESSAGE_PTR)data;
	sprintf(uin, "%d", c_mesg->uin);
	sprintf(nick, "%s", contact_list_get_nickname(CONTACT_LIST(contact_list), c_mesg->uin));

	if(!strcmp(nick, "Unknown"))
	{
		contact_list_update_user(CONTACT_LIST(contact_list),
			c_mesg->uin, uin, STATUS_NOT_IN_LIST);
	}

	if(c_mesg->type == 0x0004)
	{
		msgbox = url_new(RECEIVE_URL, -1, NULL);
		labeled_entry_set_text(LABELED_ENTRY(URL(msgbox)->uin), uin);
		labeled_entry_set_text(LABELED_ENTRY(URL(msgbox)->nick), nick);
		gtk_widget_show(msgbox);
		gtk_entry_set_text(GTK_ENTRY(URL(msgbox)->url), c_mesg->url);
		gtk_text_insert(GTK_TEXT(URL(msgbox)->message), NULL, 
			NULL, NULL, c_mesg->msg, -1);
	}
	else
	{
		msgbox = message_new(RECEIVE_MESSAGE, -1, NULL);
		labeled_entry_set_text(LABELED_ENTRY(MESSAGE(msgbox)->uin), uin);
		labeled_entry_set_text(LABELED_ENTRY(MESSAGE(msgbox)->nick), nick);
		gtk_widget_show(msgbox);
		gtk_text_insert(GTK_TEXT(MESSAGE(msgbox)->message), NULL, 
			NULL, NULL, c_mesg->msg, -1);
	}
	
}


static void EventChangeStatus(void* data)
{
	USER_UPDATE_PTR user_update;
	user_update = (USER_UPDATE_PTR)data;
	
	contact_list_update_user(CONTACT_LIST(contact_list), 
		user_update->uin, user_update->nick, user_update->status);
}


static void EventSearchResults(void* data)
{
	GtkWidget* sr_box;
	
	/* for some reason this gives me a warning if I don't cast it */
	sr_box = GTK_WIDGET(search_results_new(data));
	gtk_widget_show(sr_box);
}


static void EventInfo(void* data)
{
	SEARCH_RESULT sr;
	user_info = *(USER_INFO_PTR)data;

	if(user_info.uin != pending_search)
	{
		ICQ_Get_Ext_Info(user_info.uin);
		return;
	}
	
	sr.uin = user_info.uin;
	strcpy(sr.nick, user_info.nick);
	strcpy(sr.first, user_info.first);
	strcpy(sr.last, user_info.last);
	strcpy(sr.email, user_info.email);
	sr.auth_required = user_info.auth_required;
	sr.next = NULL;
	
	EventSearchResults(&sr);
	
	pending_search = 0;
}


static void EventExtInfo(void* data)
{
	GtkWidget* infobox;
	char uin[20];
	
	user_ext_info = *(USER_EXT_INFO_PTR)data;

	sprintf(uin, "%d", user_info.uin);

	/* for some reason this gives me a warning if I don't cast it */
	infobox = GTK_WIDGET(info_box_new());

	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->nick), user_info.nick);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->uin), uin);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->first), user_info.first);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->last), user_info.last);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->email), user_info.email);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->age), user_ext_info.age);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->sex), user_ext_info.sex);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->homepage), user_ext_info.url);
	
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->city), user_ext_info.city);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->state), user_ext_info.state);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->country), user_ext_info.country);
	labeled_entry_set_text(LABELED_ENTRY(INFO_BOX(infobox)->phone), user_ext_info.phone);
	
	gtk_widget_show(infobox);
	gtk_widget_realize(GTK_WIDGET(INFO_BOX(infobox)->info));

	gtk_text_insert(GTK_TEXT(INFO_BOX(infobox)->info), NULL, NULL, NULL, user_ext_info.about, -1);
}


void RegisterCallbacks()
{
	ICQ_Register_Callback(EVENT_LOGIN, EventLogin);
	ICQ_Register_Callback(EVENT_DISCONNECT, EventDisconnect);
	
	ICQ_Register_Callback(EVENT_MESSAGE, EventMessage);
	ICQ_Register_Callback(EVENT_ONLINE, EventChangeStatus);
	ICQ_Register_Callback(EVENT_OFFLINE, EventChangeStatus);
	ICQ_Register_Callback(EVENT_STATUS_UPDATE, EventChangeStatus);
	ICQ_Register_Callback(EVENT_SEARCH_RESULTS, EventSearchResults);
	ICQ_Register_Callback(EVENT_INFO, EventInfo);
	ICQ_Register_Callback(EVENT_EXT_INFO, EventExtInfo);
	ICQ_Register_Callback(EVENT_SEARCH_RESULTS, EventSearchResults);
}


void AddTimers()
{
	select_id = gtk_timeout_add(100, SelectServer, NULL);
	timeout_id = gtk_timeout_add(120000, KeepAlive, NULL);
}


void RemoveTimers()
{
	gtk_timeout_remove(select_id);
	gtk_timeout_remove(timeout_id);
}


void Shutdown()
{
	if(connection == ONLINE) ICQ_Disconnect();
	gtk_main_quit();
}


gint ChangeStatus(gpointer data)
{
	GtkWidget* login_window;
	int fd, error;

	if(connection == OFFLINE && (int)data < 5)
	{
		connection = CONNECTING;
		
		if(ICQ_Connect() == 0)
		{
			login_window = login_new();
			gtk_widget_show(login_window);
		}
		else
		{
			AddTimers();
		}
	}
	else if(connection == ONLINE && (int)data == 5)
	{
		ICQ_Disconnect();
		gtk_clist_clear(GTK_CLIST(CONTACT_LIST(contact_list)->online_list));
		gtk_clist_clear(GTK_CLIST(CONTACT_LIST(contact_list)->offline_list));
		gtk_clist_clear(GTK_CLIST(CONTACT_LIST(contact_list)->nil_list));
		gtk_widget_hide(CONTACT_LIST(contact_list)->nil_toggle);
		gtk_widget_hide(CONTACT_LIST(contact_list)->nil_list);
		RemoveTimers();
		connection = OFFLINE;
		return TRUE;
	}
	
	switch((int)data)
	{
		case 0: ICQ_Change_Status(STATUS_ONLINE); break;
		case 1: ICQ_Change_Status(STATUS_AWAY); break;
		case 2: ICQ_Change_Status(STATUS_NA); break;
		case 3: ICQ_Change_Status(STATUS_OCCUPIED); break;
		case 4: ICQ_Change_Status(STATUS_DND); break;
	}

	return TRUE;
}


void StartSearch()
{
	GtkWidget* srchbox;

	srchbox = search_box_new();
	gtk_widget_show(srchbox);
}
