#include <qdir.h>
#include <kapp.h>
#include <kiconloader.h>
#include <ksimpleconfig.h>
#include <qmsgbox.h>
#include <qstrlist.h>
#include <qregexp.h>
#include <sys/stat.h>  // for mkdir()
#include <unistd.h>    // for unlink()
#include "mainwindow.h"
#include "recvwindow.h"

#include "mainwindow.moc"

MainWindow::MainWindow() : KTopLevelWidget()
{
  KIconLoader *il = kapp->getIconLoader();
  resize(150,250);
  panelwidget = new QWidget(this);
  setView(panelwidget);
  listwidget = new UserList(panelwidget);

  addbutton = new QPushButton("Search/Add Users",panelwidget);
  connect(addbutton, SIGNAL(clicked()), SLOT(addCommand()));

  syswidget = new SystemBar(panelwidget);

  menubutton = new QPushButton("ICQ",panelwidget);
  connect(menubutton, SIGNAL(clicked()), SLOT(menuCommand()));
  statebutton = new ConnectButton(panelwidget);
  connect(statebutton, SIGNAL(clicked()), SLOT(stateCommand()));

  statemenu = new QPopupMenu();
  statemenu->insertItem(il->loadIcon("online.xpm"),"Online/Connect",stOnline);
  statemenu->insertItem(il->loadIcon("away.xpm"),"Away",stAway);
  statemenu->insertItem(il->loadIcon("na.xpm"),"N/A (Extended Away)",stNA);
  statemenu->insertItem(il->loadIcon("occupied.xpm"),"Occupied (Urgent Msgs)",stOccupied);
  statemenu->insertItem(il->loadIcon("dnd.xpm"),"DND (Do not Disturb)",stDND);
  statemenu->insertItem(il->loadIcon("privacy.xpm"),"Privacy (Invisible)",stInvisible);
  statemenu->insertItem(il->loadIcon("offline.xpm"),"Offline/Disconnect",stOffline);
  statemenu->setMouseTracking(TRUE);
  connect(statemenu, SIGNAL(activated(int)), SLOT(setStatus(int)));
  
  icqmenu = new QPopupMenu();
  icqmenu->insertItem(il->loadIcon("search.xpm"),"Search/Add Users",this,SLOT(addCommand()));
  icqmenu->insertSeparator();
  icqmenu->insertItem(il->loadIcon("setup.xpm"),"Preferences",MENU_PREFERENCES);
  icqmenu->insertSeparator();
  icqmenu->insertItem(il->loadIcon("shutdown.xpm"),"Shut Down",MENU_SHUTDOWN);
  icqmenu->setMouseTracking(TRUE);
  connect(icqmenu, SIGNAL(activated(int)), SLOT(icqCommand(int)));

  prefwin = new Preferences();
  if(!checkConfig())
  {
    prefwin->setUser(0,"","");
    prefwin->setServer("icq1.mirabilis.com",4000);
    prefwin->setOther(TRUE, 0, "");
    if(prefwin->exec()==QDialog::Rejected)
      exit(1);
  }

  appdir = kapp->localkdedir()+"/share/apps";
  mkdir((const char*)appdir,0700);
  appdir += "/kicq";
  mkdir((const char*)appdir,0700);
  contdir = appdir + "/contacts";
  mkdir((const char*)contdir,0700);

  findwin = new FindWindow();

  logged = FALSE;
	log_status = stOffline;
  
  loadConfig();
  setTitle();
  initContact();

  setupGUICallbacks();
}

MainWindow::~MainWindow()
{
  if(findwin)
    delete findwin;
  if(prefwin)
    delete prefwin;
  if(listwidget)
    delete listwidget;
  if(addbutton)
    delete addbutton;
  if(syswidget)
    delete syswidget;
  if(menubutton)
    delete menubutton;
  if(statebutton)
    delete statebutton;
  if(statemenu)
  {
    statemenu->clear();
    delete statemenu;
  }
  if(icqmenu)
  {
    icqmenu->clear();
    delete icqmenu;
  }
  if(panelwidget)
    delete panelwidget;
}

void MainWindow::setTitle()
{
  QString title;
  title.setNum(uin);
  setCaption(title);
}

void MainWindow::initContact()
{
  UserItem *useritem;
  ContactItem item;
  unsigned i;
	unsigned long cur_uin = 0;
	char buf[1024] = "\0";
  QString contfile, uinstr, aboutfname, notesfname;

  if(listwidget->currentItem()>=0)
    cur_uin = listwidget->getItem(listwidget->currentItem())->UIN();
  listwidget->clear();
	icq_ContClear();
  QStrList usrlist = getDirList();
  for(i=0; i<usrlist.count(); i++)
  {
    contfile = contdir+"/"+usrlist.at(i);
    KSimpleConfig usrcfg((const char*)contfile);
    usrcfg.setGroup("Info");
    uinstr = usrlist.at(i);
    uinstr.resize(uinstr.find(".")+1);
    item.uin = strtoul(uinstr, 0, 10);
    item.nick = usrcfg.readEntry("Nick", "N/A");
    item.email = usrcfg.readEntry("EMail", "N/A");
    item.firstName = usrcfg.readEntry("FirstName", "N/A");
    item.lastName = usrcfg.readEntry("LastName", "N/A");
    item.city = usrcfg.readEntry("City", "N/A");
    item.state = usrcfg.readEntry("State", "N/A");
    item.zip = usrcfg.readEntry("Zip", "N/A");
    item.country = usrcfg.readNumEntry("Country", 0);
		item.countryStat = usrcfg.readNumEntry("CountryStatus", 0);
    item.homePage = usrcfg.readEntry("HomePage", "N/A");
    item.age = usrcfg.readUnsignedNumEntry("Age", 0);
    item.phone = usrcfg.readEntry("Phone", "N/A");
    item.gender = usrcfg.readUnsignedNumEntry("Gender", 0);
		aboutfname = contdir+"/"+uinstr+".about";
		QFile aboutfile(aboutfname);
		if(aboutfile.exists())
		{
		  aboutfile.open(IO_ReadOnly);
		  aboutfile.readBlock(buf, aboutfile.size());
			buf[aboutfile.size()] = 0;
		  item.about = (const char*)buf;
			aboutfile.close();
		}
		else
			item.about = "";
		notesfname = contdir+"/"+uinstr+".notes";
		QFile notesfile(notesfname);
		if(notesfile.exists())
		{
		  notesfile.open(IO_ReadOnly);
		  notesfile.readBlock(buf, notesfile.size());
			buf[notesfile.size()] = 0;
		  item.notes = (const char*)buf;
			notesfile.close();
		}
		else
		  item.notes = "";
    item.status = (unsigned long)STATUS_OFFLINE;
    icq_ContAddUser(item.uin);
    useritem = new UserItem(item);
    listwidget->appendItem(useritem);
  }
  listwidget->changeList();
  if(cur_uin)
	  for(i=0; i<listwidget->count(); i++)
      if(listwidget->getItem(i)->UIN()==cur_uin)
        listwidget->setCurrentItem(i);
  if(isLogged())
    icq_SendContactList();
}

void MainWindow::updateContact()
{
  ContactItem *item;
  unsigned i;
  QString contfile, uinstr, aboutfname, notesfname;
	char buf[1024] = "\0";

  QStrList usrlist = getDirList();
  for(i=0; i<usrlist.count(); i++)
  {
    contfile = contdir+"/"+usrlist.at(i);
    KSimpleConfig usrcfg((const char*)contfile);
    usrcfg.setGroup("Info");
    uinstr = usrlist.at(i);
    uinstr.resize(uinstr.find(".")+1);
		item = listwidget->getInfoByUIN(strtoul(uinstr, 0, 10));
		if(item)
		{
      item->nick = usrcfg.readEntry("Nick", "N/A");
      item->email = usrcfg.readEntry("EMail", "N/A");
      item->firstName = usrcfg.readEntry("FirstName", "N/A");
      item->lastName = usrcfg.readEntry("LastName", "N/A");
      item->city = usrcfg.readEntry("City", "N/A");
			item->state = usrcfg.readEntry("State", "N/A");
			item->zip = usrcfg.readEntry("Zip", "N/A");
			item->country = usrcfg.readNumEntry("Country", 0);
			item->countryStat = usrcfg.readNumEntry("CountryStatus", 0);
			item->homePage = usrcfg.readEntry("HomePage", "N/A");
			item->age = usrcfg.readUnsignedNumEntry("Age", 0);
			item->phone = usrcfg.readEntry("Phone", "N/A");
			item->gender = usrcfg.readUnsignedNumEntry("Gender", 0);
		  aboutfname = contdir+"/"+uinstr+".about";
		  QFile aboutfile(aboutfname);
			if(aboutfile.exists())
			{
		    aboutfile.open(IO_ReadOnly);
		    aboutfile.readBlock(buf, aboutfile.size());
				buf[aboutfile.size()] = 0;
		    item->about = (const char*)buf;
				aboutfile.close();
			}
			else
			  item->about = "";
		  notesfname = contdir+"/"+uinstr+".notes";
		  QFile notesfile(notesfname);
			if(notesfile.exists())
			{
		    notesfile.open(IO_ReadOnly);
		    notesfile.readBlock(buf, notesfile.size());
				buf[notesfile.size()] = 0;
		    item->notes = (const char*)buf;
				notesfile.close();
			}
			else
			  item->notes = "";
		}
  }
  listwidget->changeList();
}

int MainWindow::checkConfig()
{
  KConfig *config = kapp->getConfig();
  config->setGroup("User");
  unsigned long uin = config->readUnsignedLongNumEntry("UIN");
  if(!uin)
    return FALSE;
  else
    return TRUE;
}

void MainWindow::loadConfig()
{
  KConfig *config = kapp->getConfig();
  config->setGroup("User");
  uin = config->readUnsignedLongNumEntry("UIN");
  passwd = config->readEntry("Password","");
  nick = config->readEntry("Nick","");
  config->setGroup("Server");
  server = config->readEntry("Host","");
  port = config->readNumEntry("Port");
  config->setGroup("Other");
  icq_Russian = config->readNumEntry("Koi2win", TRUE);
  config->setGroup("Log");
	icq_LogLevel = config->readNumEntry("Level", 0);
	logfile = config->readEntry("File","");
}

void MainWindow::closeEvent(QCloseEvent *)
{
  kapp->quit();
}

void MainWindow::resizeEvent(QResizeEvent *re)
{
  KTopLevelWidget::resizeEvent(re);
  menubutton->setGeometry(0,panelwidget->height()-25,40,25);
  statebutton->setGeometry(42,panelwidget->height()-25,panelwidget->width()-42,25);
  syswidget->setGeometry(0,menubutton->y()-syswidget->itemHeight(0)-6,panelwidget->width(),syswidget->itemHeight(0)+4);
  addbutton->setGeometry(0,syswidget->y()-22,panelwidget->width(),20);
  listwidget->setGeometry(0,0,panelwidget->width(),addbutton->y()-2);
}

void MainWindow::menuCommand()
{
  icqmenu->move(-1000,-1000);
  icqmenu->show();
  QPoint pos = menubutton->mapToGlobal(QPoint(0,0));
  pos.setY(pos.y()-icqmenu->height());
  icqmenu->popup(pos);
}

void MainWindow::stateCommand()
{
  statemenu->move(-1000,-1000);
  statemenu->show();
  QPoint pos = statebutton->mapToGlobal(QPoint(0,0));
  pos.setY(pos.y()-statemenu->height());
  statemenu->popup(pos);
}

void MainWindow::icqCommand(int num)
{
  switch(num)
  {
    case MENU_SHUTDOWN:
      kapp->quit();
      break;
    case MENU_PREFERENCES:
      prefwin->setUser(uin,nick,passwd);
      prefwin->setServer(server,port);
      prefwin->setOther(icq_Russian, icq_LogLevel, logfile);
      if(prefwin->exec()==QDialog::Accepted)
      {
        loadConfig();
				setTitle();
      }
      break;
  }
}

void MainWindow::messageRequest(int num)
{
  UserItem *usritem = listwidget->getItem(num);
  if(usritem->type()!=tpOnSeparator && usritem->type()!=tpOffSeparator && usritem->type()!=tpNILSeparator)
  {
    MsgWindow *msgwin = new MsgWindow(usritem->name(),usritem->UIN(),usritem->email());
		sendWinList.append(msgwin);
    msgwin->show();
  }
}

void MainWindow::messageRequest()
{
  if(listwidget->currentItem()>=0)
    messageRequest(listwidget->currentItem());
}

void MainWindow::messageRequest(unsigned long uin)
{
  MsgWindow *msgwin;
  ContactItem *item = listwidget->getInfoByUIN(uin);
  if(item)
    msgwin = new MsgWindow(item->nick, item->uin, item->email);
	else
    msgwin = new MsgWindow("Not In List", uin, "Not In List");
	sendWinList.append(msgwin);
	msgwin->show();
}

void MainWindow::addCommand()
{
  findwin->show();
	findwin->raise();
}

void MainWindow::offlineRequest()
{
  if(listwidget->currentItem()>=0)
	{
    UserItem *usritem = listwidget->getItem(listwidget->currentItem());
    listwidget->setStatus(usritem->UIN(),(unsigned long)STATUS_OFFLINE);
	}
}

void MainWindow::onlineRequest()
{
  if(listwidget->currentItem()>=0)
	{
    UserItem *usritem = listwidget->getItem(listwidget->currentItem());
    listwidget->setStatus(usritem->UIN(),STATUS_ONLINE);
	}
}

void MainWindow::userOnline(unsigned long uin, unsigned long status)
{
  ((MainWindow*)(kapp->topWidget()))->listwidget->setStatus(uin,status);
}

void MainWindow::userOffline(unsigned long uin)
{
  ((MainWindow*)(kapp->topWidget()))->listwidget->setStatus(uin,(unsigned long)STATUS_OFFLINE);
}

void MainWindow::infoRequest()
{
  if(listwidget->currentItem()>=0)
    infoShow(listwidget->getItem(listwidget->currentItem())->getInfo());
}

void MainWindow::infoWindow(unsigned long uin)
{
  ContactItem *item = listwidget->getInfoByUIN(uin);
	if(item)
    infoShow(*item);
}

void MainWindow::infoShow(ContactItem info, int loc)
{
  InfoWindow *ptr, *infowin = 0L;
	for(ptr=infoWinList.first(); ptr != 0L; ptr=infoWinList.next())
  {
    if(ptr->UIN()==info.uin)
		  infowin = ptr;
	}
  if(!infowin)
    infowin = new InfoWindow(loc);
	infowin->setInfo(info);
	infoWinList.append(infowin);
  infowin->show();
	infowin->raise();
}

void MainWindow::delUserRequest()
{
  UserItem *usritem;
	QString uinstr;
	if(listwidget->currentItem()>=0)
	{
    usritem = listwidget->getItem(listwidget->currentItem());
	  uinstr.setNum(usritem->UIN());
	  unlink(contdir+"/"+uinstr+".uin");
	  unlink(contdir+"/"+uinstr+".about");
	  unlink(contdir+"/"+uinstr+".notes");
		listwidget->deleteByUIN(usritem->UIN());
		listwidget->changeList();
	}
}

void MainWindow::setStatus(int status)
{
  switch(status)
	{
  	case stOffline:
      if(logged==TRUE)
      {
  			logged = FALSE;
        logout();
      }
      else
        statebutton->setStatus(stOffline);
      listwidget->allOffline();
			break;
	  case stOnline:
      if(logged==TRUE)
			{
        statebutton->setStatus(stOnline);
				icq_ChangeStatus(STATUS_ONLINE);
			}
      else
      {
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_ONLINE);
					log_status = stOnline;
				}
        else
          statebutton->setStatus(stOffline);
      }
			break;
  	case stAway:
			if(logged==TRUE)
			{
        statebutton->setStatus(stAway);
			  icq_ChangeStatus(STATUS_AWAY);
			}
			else
			{
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_AWAY);
					log_status = stAway;
				}
        else
          statebutton->setStatus(stOffline);
			}
			break;
  	case stDND:
			if(logged==TRUE)
			{
        statebutton->setStatus(stDND);
	  		icq_ChangeStatus(STATUS_DND);
			}
			else
			{
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_DND);
					log_status = stDND;
				}
        else
          statebutton->setStatus(stOffline);
			}
			break;
  	case stNA:
			if(logged==TRUE)
			{
        statebutton->setStatus(stNA);
			  icq_ChangeStatus(STATUS_NA);
			}
			else
			{
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_NA);
					log_status = stNA;
				}
        else
          statebutton->setStatus(stOffline);
			}
			break;
  	case stOccupied:
			if(logged==TRUE)
			{
        statebutton->setStatus(stOccupied);
	  		icq_ChangeStatus(STATUS_OCCUPIED);
			}
			else
			{
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_OCCUPIED);
					log_status = stOccupied;
				}
        else
          statebutton->setStatus(stOffline);
			}
			break;
  	case stInvisible:
			if(logged==TRUE)
			{
        statebutton->setStatus(stInvisible);
	  		icq_ChangeStatus(STATUS_INVISIBLE);
			}
			else
			{
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_INVISIBLE);
					log_status = stInvisible;
				}
        else
          statebutton->setStatus(stOffline);
			}
			break;
  	case stFFC:
			if(logged==TRUE)
			{
        statebutton->setStatus(stFFC);
	  		icq_ChangeStatus(STATUS_FREE_CHAT);
			}
			else
			{
        statebutton->setStatus(stLogin);
        if(icq_Connect(server,port)!=-1)
				{
          login(STATUS_FREE_CHAT);
					log_status = stFFC;
				}
        else
          statebutton->setStatus(stOffline);
			}
			break;
  	default:
      statebutton->setStatus((icqStatus)status);
			break;
	}
}

void MainWindow::setLogStatus()
{
  setStatus(log_status);
}

void MainWindow::login(unsigned long status)
{
  icq_Init(uin, (const char*)passwd);
  icq_Login(status);
  idletimer = startTimer(300);
	keepalivetimer = startTimer(120000);
}

void MainWindow::logout()
{
	stopAll();
  icq_Quit();
  killTimer(idletimer);
  idletimer = -1;
	killTimer(keepalivetimer);
	keepalivetimer = -1;
  setStatus(stOffline);
}

void MainWindow::setupGUICallbacks()
{
  icq_Logged = loggedIn;
  icq_Disconnected = disconnected;
	icq_UserOnline = userOnline;
	icq_UserOffline = userOffline;
	icq_UserStatusUpdate = userOnline;
	icq_RecvMessage = message;
	icq_InfoReply = infoReply;
	icq_ExtInfoReply = infoExtReply;
	icq_SrvAck = srvAck;
  icq_Log = logEvent;
}

void MainWindow::logEvent(time_t time, unsigned char level, const char *str)
{
  char lvl[4][4] = {"fat", "err", "wrn", "msg"};
	int std = FALSE;
	FILE *log;
	struct tm *log_time;
	log = fopen(((MainWindow*)(kapp->topWidget()))->logFile(), "a");
	if(!log)
	{
    log = fdopen(1, "w");
		std = TRUE;
	}
	log_time = localtime(&time);
  fprintf(log, "%02i/%02i/%04i %02i:%02i:%02i [%s] %s", log_time->tm_mday, log_time->tm_mon+1, log_time->tm_year+1900, log_time->tm_hour, log_time->tm_min, log_time->tm_sec, lvl[level-1], str);
	if(!std)
	  fclose(log);
}

void MainWindow::loggedIn()
{
  ((MainWindow*)(kapp->topWidget()))->logged = TRUE;
  ((MainWindow*)(kapp->topWidget()))->setLogStatus();
}

void MainWindow::disconnected()
{
  ((MainWindow*)(kapp->topWidget()))->setStatus(stOffline);
}

void MainWindow::stopAll()
{
  InfoWindow *ptr;
  findwin->stopCommand();
	for(ptr=infoWinList.first(); ptr != 0L; ptr=infoWinList.next())
  {
    ptr->stopCommand();
	}
}

void MainWindow::message(unsigned long uin, time_t time, const char *data)
{
  RecvWindow *recvwin = new RecvWindow(((MainWindow*)(kapp->topWidget()))->listwidget->getNickByUIN(uin),uin,
                                       ((MainWindow*)(kapp->topWidget()))->listwidget->getEMailByUIN(uin),time,
                                       data);
  recvwin->show();
}

/*
void MainWindow::url(URL *)
{
}

void MainWindow::added(ADDED *)
{
}
*/

void MainWindow::infoReply(unsigned long uin, const char *nick, const char *first, const char *last, const char *email, char auth)
{
  ((MainWindow*)(kapp->topWidget()))->infoReplyHandler(uin, nick, first, last, email, auth);
}

void MainWindow::infoExtReply(unsigned long uin, const char *city, unsigned short country_code, char country_stat, const char *state, unsigned short age, char gender, const char *phone, const char *hp, const char *about)
{
  ((MainWindow*)(kapp->topWidget()))->infoExtReplyHandler(uin, city, country_code, country_stat, state, age, gender, phone, hp, about);
}

void MainWindow::infoReplyHandler(unsigned long uin, const char *nick, const char *first, const char *last, const char *email, char auth)
{
  InfoWindow *ptr;
	for(ptr=infoWinList.first(); ptr != 0L; ptr=infoWinList.next())
  {
    if(ptr->UIN()==uin)
      ptr->updateInfo(nick, first, last, email, auth);
	}
}

void MainWindow::infoExtReplyHandler(unsigned long uin, const char *city, unsigned short country_code, char country_stat, const char *state, unsigned short age, char gender, const char *phone, const char *hp, const char *about)
{
  InfoWindow *ptr;
	for(ptr=infoWinList.first(); ptr != 0L; ptr=infoWinList.next())
  {
    if(ptr->UIN()==uin)
      ptr->updateExtInfo(city, country_code, country_stat, state, age, gender, phone, hp, about);
	}
}

void MainWindow::srvAck(unsigned short seq)
{
  ((MainWindow*)(kapp->topWidget()))->closeSendWin(seq);
}

void MainWindow::closeSendWin(unsigned short seq)
{
  MsgWindow *ptr;
	for(ptr=sendWinList.first(); ptr != 0L; ptr=sendWinList.next())
  {
    if(ptr->seq()==seq)
		{
      ptr->hide();
			delSendWin(ptr);
		}
	}
}

void MainWindow::timerEvent(QTimerEvent *te)
{
  if(te->timerId()==idletimer)
    icq_Main();
	if(te->timerId()==keepalivetimer)
		icq_KeepAlive();
}

void MainWindow::saveConfig(unsigned long uin, const char *nick, const char *pass, const char *host, int port, int rus, int loglvl, const char *logfile)
{
  KConfig *config = kapp->getConfig();
  config->setGroup("User");
  config->writeEntry("UIN",uin);
  config->writeEntry("Nick",nick);
  config->writeEntry("Password",pass);
  config->setGroup("Server");
  config->writeEntry("Host",host);
  config->writeEntry("Port",port);
  config->setGroup("Other");
  icq_Russian = rus;
  config->writeEntry("Koi2win",icq_Russian);
	config->setGroup("Log");
	icq_LogLevel = loglvl;
	config->writeEntry("Level",loglvl);
	config->writeEntry("File",logfile);
  config->sync();
}

UserList *MainWindow::listWidget()
{
  return listwidget;
}

FindWindow *MainWindow::findWindow()
{
  return findwin;
}

int MainWindow::isLogged()
{
  return logged;
}

QStrList MainWindow::getDirList()
{
  QDir dir(contdir);
  return *dir.entryList("*.uin", QDir::Files | QDir::Readable);
}

void MainWindow::saveUser(unsigned long uin, QString nick, QString first, QString last, QString email, unsigned char auth)
{
  QString uinstr, uinfile;
  uinstr.setNum(uin);
  uinfile = contdir+"/"+uinstr+".uin";
  KSimpleConfig usrcfg((const char*)uinfile);
  usrcfg.setGroup("Info");
  usrcfg.writeEntry("Nick", nick);
  usrcfg.writeEntry("FirstName", first);
  usrcfg.writeEntry("LastName", last);
  usrcfg.writeEntry("EMail", email);
  usrcfg.sync();
}

void MainWindow::saveUser(ContactItem item)
{
  QString uinstr, uinfile, aboutfname, notesfname;
  uinstr.setNum(item.uin);
  uinfile = contdir+"/"+uinstr+".uin";
  KSimpleConfig usrcfg((const char*)uinfile);
  usrcfg.setGroup("Info");
  usrcfg.writeEntry("Nick", item.nick);
  usrcfg.writeEntry("FirstName", item.firstName);
  usrcfg.writeEntry("LastName", item.lastName);
  usrcfg.writeEntry("EMail", item.email);
	usrcfg.writeEntry("City",item.city);
	usrcfg.writeEntry("State",item.state);
	usrcfg.writeEntry("Zip",item.zip);
	usrcfg.writeEntry("Country",item.country);
	usrcfg.writeEntry("CountryStatus",item.countryStat);
	usrcfg.writeEntry("HomePage",item.homePage);
	usrcfg.writeEntry("Age",item.age);
	usrcfg.writeEntry("Phone",item.phone);
	usrcfg.writeEntry("Gender",item.gender);
	aboutfname = contdir+"/"+uinstr+".about";
	QFile aboutfile(aboutfname);
	if(aboutfile.exists())
	  aboutfile.remove();
	aboutfile.open(IO_WriteOnly | IO_Raw);
	aboutfile.writeBlock((const char*)item.about, item.about.length());
	notesfname = contdir+"/"+uinstr+".notes";
	QFile notesfile(notesfname);
	if(notesfile.exists())
    notesfile.remove();
	notesfile.open(IO_WriteOnly | IO_Raw);
	notesfile.writeBlock((const char*)item.notes, item.notes.length());
  usrcfg.sync();
}

void MainWindow::delInfoWin(InfoWindow *win)
{
  infoWinList.removeRef(win);
}

void MainWindow::delSendWin(MsgWindow *win)
{
  sendWinList.removeRef(win);
}

const char *MainWindow::logFile()
{
  return (const char*)logfile;
}
