#ifndef _MAINWINDOW_
#define _MAINWINDOW_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ktopwidget.h>
#include <kmenubar.h>
#include <ktoolbar.h>
#include <qlistbox.h>
#include <qwidget.h>
#include <qpushbt.h>
#include <icq.h>
#include "userlist.h"
#include "useritem.h"
#include "systembar.h"
#include "connectbutton.h"
#include "preferences.h"
#include "findwindow.h"
#include "infowindow.h"
#include "msgwindow.h"

#define MENU_SHUTDOWN		1
#define MENU_PREFERENCES	2

class MainWindow : public KTopLevelWidget {
  Q_OBJECT
public:
  MainWindow();
  ~MainWindow();
  void loadConfig();
  void saveConfig(unsigned long, const char *, const char *, const char *, int, int, int, const char*);
  UserList *listWidget();
  FindWindow *findWindow();
  int isLogged();
  void saveUser(unsigned long, QString, QString, QString, QString, unsigned char);
	void saveUser(ContactItem);
  void initContact();
	void updateContact();
	void infoWindow(unsigned long);
	void infoShow(ContactItem, int loc = TRUE);
	void delInfoWin(InfoWindow*);
	void delSendWin(MsgWindow*);
	void infoReplyHandler(unsigned long, const char*, const char*, const char*, const char*, char);
	void infoExtReplyHandler(unsigned long, const char*, unsigned short, char, const char*, unsigned short, char, const char*, const char*, const char*);
public slots:
  void menuCommand();
  void stateCommand();
  void messageRequest(int);
  void messageRequest();
	void messageRequest(unsigned long);
  void addCommand();
  void setStatus(int);
	void setLogStatus();
  void onlineRequest();
  void offlineRequest();
  void icqCommand(int);
  void infoRequest();
	void delUserRequest();
	const char *logFile();
signals:
protected:
  int logged;
  int idletimer, keepalivetimer;
  unsigned long uin;
  QString passwd;
  QString nick;
  QString server;
  unsigned short port;
	unsigned long log_status;
  QWidget *panelwidget;
  UserList *listwidget;
  SystemBar *syswidget;
  QPushButton *menubutton;
  ConnectButton *statebutton;
  QPushButton *addbutton;
  QPopupMenu *statemenu;
  QPopupMenu *icqmenu;
  Preferences *prefwin;
  FindWindow *findwin;
  QString appdir, contdir;
	QList<InfoWindow> infoWinList;
	QList<MsgWindow> sendWinList;
	QString logfile;
  void resizeEvent(QResizeEvent *);
  void closeEvent(QCloseEvent *);
  void setupGUICallbacks();
  static void loggedIn();
  static void disconnected();
  static void logEvent(time_t, unsigned char, const char*);
	static void userOnline(unsigned long, unsigned long);
	static void userOffline(unsigned long);
	static void message(unsigned long, time_t, const char*);
	static void infoReply(unsigned long, const char*, const char*, const char*, const char*, char);
	static void infoExtReply(unsigned long, const char*, unsigned short, char, const char*, unsigned short, char, const char*, const char*, const char*);
	static void srvAck(unsigned short);
/*  static void url(URL *);
	static void added(ADDED *); */
  virtual void timerEvent(QTimerEvent *);
  void login(unsigned long);
  void logout();
  int checkConfig();
  void setTitle();
  QStrList getDirList();
	void stopAll();
	void closeSendWin(unsigned short);
protected slots:
private:
private slots:
};
#endif /* _MAINWINDOW_ */
