#include <kapp.h>
#include <kiconloader.h>
#include <qmsgbox.h>
#include <qmovie.h>
#include "msgwindow.h"
#include "mainwindow.h"

#include "msgwindow.moc"

MsgWindow::MsgWindow(const char *n, unsigned long u, const char *em) : QDialog()
{
  KIconLoader *il = kapp->getIconLoader();
  setCaption("Send Message");
  tobox = new QGroupBox("To",this);
  sendbutton = new QPushButton("Send",this);
  connect(sendbutton, SIGNAL(clicked()),SLOT(sendMessage()));
  cancelbutton = new QPushButton("Cancel",this);
  connect(cancelbutton, SIGNAL(clicked()),SLOT(cancelMessage()));
  uinline = new QLabel(this);
  uinline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  uinlabel = new QLabel(uinline,"ICQ#:",this);
  handleline = new QLabel(this);
  handleline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  handlelabel = new QLabel(handleline,"Handle:",this);
  emailline = new QLabel(this);
  emailline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  emaillabel = new QLabel(emailline,"EMail:",this);
  infobutton = new QPushButton(this);
  infobutton->setAutoResize(TRUE);
  infobutton->setPixmap(il->loadIcon("info.xpm"));
	connect(infobutton, SIGNAL(clicked()), SLOT(infoRequest()));
  editbox = new QMultiLineEdit(this);
  editlabel = new QLabel(editbox,"Enter Message:",this);
  updateanim = new QLabel(this);
  QString filename = kapp->kde_datadir()+"/"+kapp->appName()+"/pics/search.gif";
  updateanim->setMovie(QMovie((const char*)filename));
  updateanim->movie()->pause();
  setName(n);
  setUIN(u);
  setEMail(em);
  editbox->setFocus();
  resize(500,250);
	sendSeq = -1;
}

MsgWindow::~MsgWindow()
{
  if(sendbutton)
    delete sendbutton;
  if(cancelbutton)
    delete cancelbutton;
  if(tobox)
    delete tobox;
  if(uinlabel)
    delete uinlabel;
  if(handlelabel)
    delete handlelabel;
  if(emaillabel)
    delete emaillabel;
  if(uinline)
    delete uinline;
  if(handleline)
    delete handleline;
  if(emailline)
    delete emailline;
  if(infobutton)
    delete infobutton;
  if(editbox)
    delete editbox;
  if(editlabel)
    delete editlabel;
}

void MsgWindow::resizeEvent(QResizeEvent *re)
{
  QDialog::resizeEvent(re);
  QSize uinlblsize,handlelblsize,emaillblsize,toboxsize,editlblsize,btn1size,btn2size;
  toboxsize.setWidth(width()-6);
  uinlblsize = uinlabel->sizeHint();
  handlelblsize = handlelabel->sizeHint();
  emaillblsize = emaillabel->sizeHint();
  int w = (toboxsize.width()-uinlblsize.width()-handlelblsize.width()-emaillblsize.width()-75)/4;
  uinline->resize(w,uinline->sizeHint().height());
  handleline->resize(w,uinline->sizeHint().height());
  emailline->resize(w*2-7,uinline->sizeHint().height());
  uinlabel->resize(uinlblsize.width(),uinline->height());
  handlelabel->resize(handlelblsize.width(),handleline->height());
  emaillabel->resize(emaillblsize.width(),emailline->height());
  uinlabel->move(12,15);
  uinline->move(3+uinlabel->x()+uinlabel->width(),15);
  handlelabel->move(12+uinline->x()+uinline->width(),15);
  handleline->move(3+handlelabel->x()+handlelabel->width(),15);
  emaillabel->move(12+handleline->x()+handleline->width(),15);
  emailline->move(3+emaillabel->x()+emaillabel->width(),15);
  tobox->setGeometry(3,3,toboxsize.width(),uinline->height()+19);
  infobutton->move(10+emailline->x()+emailline->width(),15+(uinline->height()-infobutton->height())/2);
  btn1size = sendbutton->sizeHint();
  btn2size = cancelbutton->sizeHint();
  int h = QMAX(btn1size.height(),btn2size.height());
  w = QMAX(QMAX(btn1size.width(),btn2size.width()),75);
  sendbutton->setGeometry(width()/2-w-6,height()-h-12,w,h);
  cancelbutton->setGeometry(width()/2+6,height()-h-12,w,h);
  editlblsize = editlabel->sizeHint();
  editlabel->setGeometry(6,8+tobox->height(),editlblsize.width(),editlblsize.height());
  editbox->setGeometry(3,editlabel->y()+editlabel->height()+4,width()-6,height()-editlabel->y()-editlabel->height()-h-28);
  updateanim->setGeometry(width()-40,height()-40,32,32);
  updateanim->movie()->restart();
}

void MsgWindow::setUIN(unsigned long u)
{
  uin = u;
  QString text;
  text.setNum(uin);
  uinline->setText(text);
}

void MsgWindow::setName(const char *name)
{
  handleline->setText(name);
}

void MsgWindow::setEMail(const char *email)
{
  emailline->setText(email);
}

void MsgWindow::sendMessage()
{
  if(((MainWindow*)(kapp->topWidget()))->isLogged())
	{
    sendSeq = icq_SendMessage(uin,editbox->text());
		updateanim->movie()->unpause();
		sendbutton->setEnabled(FALSE);
	}
  else
    QMessageBox::warning(this, "Error", "You are not connected to the icq network!");
}

void MsgWindow::cancelMessage()
{
  if(sendSeq==-1)
	{
    hide();
	  ((MainWindow*)(kapp->topWidget()))->delSendWin(this);
	}
	else
	{
    updateanim->movie()->pause();
		sendbutton->setEnabled(TRUE);
		sendSeq = -1;
	}
}

void MsgWindow::closeEvent(QCloseEvent *ce)
{
  QDialog::closeEvent(ce);
	((MainWindow*)(kapp->topWidget()))->delSendWin(this);
}

void MsgWindow::infoRequest()
{
  ContactItem tmpitem, *item = ((MainWindow*)(kapp->topWidget()))->listWidget()->getInfoByUIN(uin);
	if(item)
	  ((MainWindow*)(kapp->topWidget()))->infoWindow(uin);
	else
	{
    tmpitem.uin = uin;
		((MainWindow*)(kapp->topWidget()))->infoShow(tmpitem, FALSE);
	}
}

long MsgWindow::seq()
{
  return sendSeq;
}
