#include <kapp.h>
#include <kiconloader.h>
#include <qpalette.h>
#include "recvwindow.h"
#include "mainwindow.h"

#include "recvwindow.moc"

RecvWindow::RecvWindow(const char *name, unsigned long uin, const char *email, time_t time, const char *msg) : QDialog()
{
  KIconLoader *il = kapp->getIconLoader();
  setCaption("Incoming Message");
  frombox = new QGroupBox("From",this);
  handleline = new QLabel(this);
  handleline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  handlelabel = new QLabel(handleline,"Handle:",this);
  uinline = new QLabel(this);
  uinline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  uinlabel = new QLabel(uinline,"ICQ#:",this);
  emailline = new QLabel(this);
  emailline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  emaillabel = new QLabel(emailline,"EMail:",this);
  dateline = new QLabel(this);
  dateline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  datelabel = new QLabel(uinline,"Date:",this);
  timeline = new QLabel(this);
  timeline->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  timelabel = new QLabel(uinline,"Time:",this);
  infobutton = new QPushButton(this);
  infobutton->setAutoResize(TRUE);
  infobutton->setPixmap(il->loadIcon("info.xpm"));
	connect(infobutton, SIGNAL(clicked()), SLOT(infoRequest()));
  msgbox = new QGroupBox("Message",this);
  msgedit = new QMultiLineEdit(this);
  msgedit->setEnabled(FALSE);
  QPalette pal = msgedit->palette();
  QColorGroup colgrp = pal.disabled();
  pal.setDisabled(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),colgrp.dark(),colgrp.mid(),colgrp.foreground(),colgrp.base()));
  msgedit->setPalette(pal);
  respbox = new QGroupBox("Response Action:",this);
  addbtn = new QPushButton("Add To Contact List",this);
	connect(addbtn, SIGNAL(clicked()), SLOT(addRequest()));
  replybtn = new QPushButton("Reply",this);
	connect(replybtn, SIGNAL(clicked()), SLOT(replyRequest()));
  closebtn = new QPushButton("Close",this);
  connect(closebtn, SIGNAL(clicked()), SLOT(closeBtn()));
  nextbtn = new QPushButton("Read Next",this);
  nextbtn->setEnabled(FALSE);			// Not working yet!
  setName(name);
  setUIN(uin);
  setEMail(email);
  setTime(time);
  setMessage(msg);
	if(((MainWindow*)kapp->topWidget())->listWidget()->getInfoByUIN(uin))
    addbtn->setEnabled(FALSE);
  replybtn->setFocus();
  resize(400,300);
}

RecvWindow::~RecvWindow()
{
  if(replybtn)
    delete replybtn;
  if(closebtn)
    delete closebtn;
  if(nextbtn)
    delete nextbtn;
  if(addbtn)
    delete addbtn;
  if(frombox)
    delete frombox;
  if(msgbox)
    delete msgbox;
  if(respbox)
    delete respbox;
  if(uinlabel)
    delete uinlabel;
  if(handlelabel)
    delete handlelabel;
  if(emaillabel)
    delete emaillabel;
  if(datelabel)
    delete datelabel;
  if(timelabel)
    delete timelabel;
  if(uinline)
    delete uinline;
  if(handleline)
    delete handleline;
  if(emailline)
    delete emailline;
  if(dateline)
    delete dateline;
  if(timeline)
    delete timeline;
  if(infobutton)
    delete infobutton;
  if(msgedit)
    delete msgedit;
}

void RecvWindow::resizeEvent(QResizeEvent *re)
{
  QSize handlelblsize, handlelinesize, uinlblsize, uinlinesize, emaillblsize, emaillinesize,
        datelblsize, datelinesize, timelblsize, timelinesize, fromboxsize, closebtnsize, nextbtnsize,
        replybtnsize, addbtnsize;
  QDialog::resizeEvent(re);
  fromboxsize.setWidth(width()-6);
  handlelblsize = handlelabel->sizeHint();
  handlelinesize = handleline->sizeHint();
  uinlblsize = uinlabel->sizeHint();
  uinlinesize = uinline->sizeHint();
  emaillblsize = emaillabel->sizeHint();
  emaillinesize = emailline->sizeHint();
  datelblsize = datelabel->sizeHint();
  datelinesize = dateline->sizeHint();
  timelblsize = timelabel->sizeHint();
  timelinesize = timeline->sizeHint();
  int h = QMAX(QMAX(QMAX(QMAX(handlelblsize.height(),handlelinesize.height()),QMAX(uinlblsize.height(),uinlinesize.height())),
          QMAX(QMAX(emaillblsize.height(),emaillinesize.height()),QMAX(datelblsize.height(),datelinesize.height()))),
          QMAX(timelblsize.height(),timelinesize.height()));
  int w1 = QMAX(handlelblsize.width(),uinlblsize.width());
  handlelabel->resize(w1,h);
  uinlabel->resize(w1,h);
  int w2 = QMAX(QMAX(emaillblsize.width(),datelblsize.width()),timelblsize.width());
  emaillabel->resize(w2,h);
  datelabel->resize(w2,h);
  timelabel->resize(w2,h);
  int w = (int)((fromboxsize.width()-70)/2.6);
  int we = fromboxsize.width()-70-w;
  handlelabel->move(12,15);
  uinlabel->move(12,handlelabel->y()+handlelabel->height()+5);
  handleline->setGeometry(handlelabel->x()+w1+5,handlelabel->y(),w-w1,h);
  uinline->setGeometry(handleline->x(),uinlabel->y(),w-w1,h);
  emaillabel->move(w+25,handlelabel->y());
  datelabel->move(w+25,uinlabel->y());
  emailline->setGeometry(emaillabel->x()+w2+5,emaillabel->y(),we-w2,h);
  int w4 = (int)((we-w2*2-10)/2.6);
  int w3 = we-w2*2-10-w4;
  dateline->setGeometry(emailline->x(),datelabel->y(),w3,h);
  timelabel->move(w+w3+datelabel->width()+40,datelabel->y());
  timeline->setGeometry(emailline->x()+emailline->width()-w4,timelabel->y(),w4,h);
  frombox->setGeometry(3,3,fromboxsize.width(),h*2+24);
  infobutton->move(frombox->x()+frombox->width()-infobutton->width()-12,emailline->y()+(emailline->height()-infobutton->height())/2);
  closebtnsize = closebtn->sizeHint();
  nextbtnsize = nextbtn->sizeHint();
  h = QMAX(closebtnsize.height(),nextbtnsize.height());
  w = QMAX(closebtnsize.width(),nextbtnsize.width());
  closebtn->setGeometry(width()-w*2-20,height()-h-6,w,h);
  nextbtn->setGeometry(width()-w-10,height()-h-6,w,h);
  msgbox->setGeometry(3,frombox->height()+6,width()-6,height()-frombox->height()-h-19);
  replybtnsize = replybtn->sizeHint();
  addbtnsize = addbtn->sizeHint();
  h = QMAX(replybtnsize.height(),addbtnsize.height());
  w = QMAX(replybtnsize.width(),addbtnsize.width());
  respbox->setGeometry(9,msgbox->y()+msgbox->height()-h-36,width()-18,h+30);
  addbtn->setGeometry(respbox->x()+(respbox->width()-w*2-10)/2,respbox->y()+18,w,h);
  replybtn->setGeometry(addbtn->x()+addbtn->width()+10,respbox->y()+18,w,h);
  msgedit->setGeometry(10,msgbox->y()+17,width()-22,msgbox->height()-respbox->height()-25);
}

void RecvWindow::setMessage(const char *message)
{
  msgedit->setText(message);
}

void RecvWindow::setUIN(unsigned long u)
{
  uin = u;
  QString text;
  text.setNum(uin);
  uinline->setText(text);
}

void RecvWindow::setName(const char *name)
{
  nick = name;
  handleline->setText(name);
}

void RecvWindow::setEMail(const char *pemail)
{
  email = pemail;
  emailline->setText(email);
}

void RecvWindow::setTime(time_t r_time)
{
  QString date;
	QString time;
	struct tm *rcv_time;
	rcv_time = localtime(&r_time);
  date.sprintf("%02i/%02i/%04i", rcv_time->tm_mday, rcv_time->tm_mon+1, rcv_time->tm_year+1900);
  dateline->setText(date);
  time.sprintf("%02i:%02i", rcv_time->tm_hour, rcv_time->tm_min);
  timeline->setText(time);
}

void RecvWindow::closeBtn()
{
  hide();
  delete this;
}

void RecvWindow::closeEvent(QCloseEvent *ce)
{
  QDialog::closeEvent(ce);
  hide();
  delete this;
}

void RecvWindow::infoRequest()
{
  ContactItem tmpitem, *item = ((MainWindow*)(kapp->topWidget()))->listWidget()->getInfoByUIN(uin);
	if(item)
	  ((MainWindow*)(kapp->topWidget()))->infoWindow(uin);
	else
	{
    tmpitem.uin = uin;
		((MainWindow*)(kapp->topWidget()))->infoShow(tmpitem, FALSE);
	}
}

void RecvWindow::addRequest()
{
  ContactItem item;
	item.uin = uin;
	item.nick.setNum(uin);
	item.email = "N/A";
	((MainWindow*)(kapp->topWidget()))->saveUser(item);
	((MainWindow*)(kapp->topWidget()))->initContact();
}

void RecvWindow::replyRequest()
{
  ((MainWindow*)(kapp->topWidget()))->messageRequest(uin);
}
