#include <kapp.h>
#include <kiconloader.h>
#include "useritem.h"
#include "mainwindow.h"

UserItem::UserItem(ContactItem item) : QListBoxItem()
{
  KIconLoader *loader = kapp->getIconLoader();
  info = item;
  itype = tpUser;
  pm_last = loader->loadIcon("gray.xpm");
	pm_stat = loader->loadIcon("null.xpm");
}

UserItem::UserItem(itType t) : QListBoxItem()
{
  itype = t;
  KIconLoader *loader = kapp->getIconLoader();
  switch(t)
  {
    case tpOnSeparator:
      info.nick = "Online";
      break;
    case tpOffSeparator:
      info.nick = "Offline";
      break;
    case tpNILSeparator:
      info.nick = "Not In List";
      break;
	  case tpSystem:
			info.nick = "System";
      pm_last = loader->loadIcon("gray.xpm");
			pm_stat = loader->loadIcon("null.xpm");
			break;
    case tpUser:
			info.nick = "Unknown";
      pm_last = loader->loadIcon("gray.xpm");
			pm_stat = loader->loadIcon("null.xpm");
			break;
  }
}

UserItem::UserItem(UserItem *item)
{
  KIconLoader *loader = kapp->getIconLoader();
  info = item->getInfo();
  itype = item->type();
  if(itype!=tpOffSeparator && itype!=tpOnSeparator && itype!=tpNILSeparator)
	  switch(info.status & 0xFFFF)
		{
      case STATUS_ONLINE:
		  case 0xFFFF:
        pm_last = loader->loadIcon("gray.xpm");
				pm_stat = loader->loadIcon("null.xpm");
				break;
      case STATUS_AWAY:
        pm_last = loader->loadIcon("gray.xpm");
        pm_stat = loader->loadIcon("uaway.xpm");
				break;
      case STATUS_NA:
        pm_last = loader->loadIcon("gray.xpm");
        pm_stat = loader->loadIcon("una.xpm");
				break;
      case STATUS_DND:
        pm_last = loader->loadIcon("gray.xpm");
        pm_stat = loader->loadIcon("udnd.xpm");
				break;
      case STATUS_OCCUPIED:
        pm_last = loader->loadIcon("gray.xpm");
        pm_stat = loader->loadIcon("uoccupied.xpm");
				break;
      case STATUS_INVISIBLE:
      case STATUS_INVISIBLE_2:
        pm_last = loader->loadIcon("gray.xpm");
        pm_stat = loader->loadIcon("uprivacy.xpm");
				break;
      default:
				printf("!!!!!!!!!!!!! - %lu\n", info.status);
				break;
		}
}

const QPixmap *UserItem::pixmap_last()
{
  return &pm_last;
}

const QPixmap *UserItem::pixmap_stat()
{
  return &pm_stat;
}

void UserItem::paint(QPainter *p)
{
  QFontMetrics fm = p->fontMetrics();
  if(itype==tpOffSeparator || itype==tpOnSeparator || itype==tpNILSeparator)
  {
    int w = ((MainWindow*)(kapp->topWidget()))->listWidget()->width()-5;
    p->setPen(QColor(white));
    p->drawLine(0,fm.lineSpacing()/2,w/2-fm.width(info.nick)/2-6,fm.lineSpacing()/2);
    p->drawLine(w/2+fm.width(info.nick)/2+5,fm.lineSpacing()/2,w,fm.lineSpacing()/2);
    p->setPen(QColor(black));
    p->drawLine(0,fm.lineSpacing()/2-1,w/2-fm.width(info.nick)/2-6,fm.lineSpacing()/2-1);
    p->drawLine(w/2+fm.width(info.nick)/2+5,fm.lineSpacing()/2-1,w,fm.lineSpacing()/2-1);
    p->drawText(0,0,w,fm.lineSpacing(),AlignCenter,info.nick);
  }
  else
  {
    p->drawPixmap(3, (QMAX(pm_last.height()+6, fm.lineSpacing())-pm_last.height())/2, pm_last);
    p->drawPixmap(3, (QMAX(pm_stat.height()+6, fm.lineSpacing())-pm_stat.height())/2, pm_stat);
    int yPos;
    if (pm_last.height() < fm.height())
      yPos = fm.ascent() + fm.leading()/2;
    else
      yPos = (pm_last.height()+6)/2 - fm.height()/2 + fm.ascent();
    p->drawText(pm_last.width() + 7, yPos, info.nick);
  }
}

int UserItem::height(const QListBox *lb ) const
{
  return QMAX(pm_last.height()+6, lb->fontMetrics().lineSpacing());
}

int UserItem::width(const QListBox *lb ) const
{
  return pm_last.width() + lb->fontMetrics().width(info.nick) + 6;
}

itType UserItem::type()
{
  return itype;
}

unsigned long UserItem::UIN()
{
  return info.uin;
}

const char *UserItem::name()
{
  return info.nick;
}

const char *UserItem::email()
{
  return info.email;
}

const char *UserItem::first()
{
  return info.firstName;
}

const char *UserItem::last()
{
  return info.lastName;
}

ContactItem UserItem::getInfo()
{
  return info;
}

ContactItem *UserItem::getInfoPtr()
{
  return &info;
}

void UserItem::setStatus(unsigned long status)
{
  info.status = status;
}

unsigned long UserItem::getStatus()
{
  return info.status;
}
