#include <qpopmenu.h>
#include <qpainter.h>
#include <kiconloader.h>
#include <kapp.h>
#include <icq.h>
#include "userlist.h"

#include "userlist.moc"

UserList::UserList(QWidget *parent, const char *name, WFlags f) : QListBox(parent, name, f)
{
  oldCurrent = 0;
  QPalette pal = palette();
  QColorGroup colgrp = pal.normal();
  pal.setNormal(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),colgrp.dark(),colgrp.mid(),colgrp.text(),colgrp.background()));
  colgrp = pal.active();
  pal.setActive(QColorGroup(colgrp.foreground(),colgrp.background(),colgrp.light(),colgrp.dark(),colgrp.mid(),colgrp.text(),colgrp.background()));
  setPalette(pal);

  connect(this, SIGNAL(highlighted(int)), SLOT(newSelect(int)));
  connect(this, SIGNAL(selected(int)), kapp->topWidget() ,SLOT(messageRequest(int)));

  KIconLoader *il = kapp->getIconLoader();
  popup = new QPopupMenu();
  popup->insertItem(il->loadIcon("message.xpm"),"Send Message", kapp->topWidget(), SLOT(messageRequest()));
  popup->insertItem(il->loadIcon("file.xpm"),"Send File (N/A)", 1); // kapp->topWidget(), SLOT(onlineRequest()));
  popup->insertSeparator();
  popup->insertItem(il->loadIcon("chat.xpm"),"ICQ Chat (N/A)", 2); //kapp->topWidget(), SLOT(offlineRequest()));
  popup->insertItem(il->loadIcon("info.xpm"),"Info", kapp->topWidget(), SLOT(infoRequest()));
  popup->insertItem(il->loadIcon("delete.xpm"),"Delete", kapp->topWidget(), SLOT(delUserRequest()));
	popup->setItemEnabled(1,FALSE);
	popup->setItemEnabled(2,FALSE);
}

UserList::~UserList()
{
  clear();
  if(popup)
  {
    popup->clear();
    delete popup;
  }
}

void UserList::focusOutEvent(QFocusEvent *e)
{
  oldCurrent = currentItem();
  QListBox::focusOutEvent(e);
  clearSelection();
}

void UserList::newSelect(int)
{
  oldCurrent = currentItem();
}

void UserList::focusInEvent(QFocusEvent *e)
{
  setCurrentItem(oldCurrent);
  QListBox::focusInEvent(e);
}

void UserList::paintCell(QPainter *p, int row, int col)
{
  UserItem *lbi = (UserItem*)item(row);
  if(!lbi)
    return;

  QColorGroup g = colorGroup();
  if(isSelected(row) && lbi->type()!=tpOnSeparator && lbi->type()!=tpOffSeparator && lbi->type()!=tpNILSeparator)
  {
    QColor fc(blue);
    p->fillRect(0, 0, cellWidth(col), cellHeight(row), fc);
    p->setPen(QColor(white));
    p->setBackgroundColor(fc);
  }
  else
  {
    p->setBackgroundColor(g.base());
    switch(lbi->getStatus() & 0xFFFF)
    {
      case 0xFFFF:
        p->setPen(QColor(red));
        break;
      case STATUS_ONLINE:
      case STATUS_AWAY:
      case STATUS_NA:
      case STATUS_DND:
      case STATUS_OCCUPIED:
      case STATUS_INVISIBLE:
      case STATUS_INVISIBLE_2:
        p->setPen(QColor(blue));
        break;
      default:
        p->setPen(g.text());
        break;
    }
  }
  lbi->paint(p);
  p->setBackgroundColor(g.base());
  p->setPen(g.text());
}

void UserList::mousePressEvent(QMouseEvent *e)
{
  QListBox::mousePressEvent(e);
  int t=getItem(currentItem())->type();
  if(t!=tpOnSeparator && t!=tpOffSeparator && t!=tpNILSeparator)
    if(e->button()==RightButton)
      popup->popup(mapToGlobal(QPoint(e->x(),e->y())));
}

UserItem *UserList::getItem(int i)
{
  return (UserItem*)item(i);
}

void UserList::setStatus(unsigned long uin, unsigned long status)
{
  UserItem *ptr, *usritem;
  for(ptr=offlineList.first(); ptr != 0L; ptr=offlineList.next())
    if(ptr->getInfo().uin==uin)
    {
      ptr->setStatus(status);
      if(status!=(unsigned long)STATUS_OFFLINE)
      {
        usritem = new UserItem(ptr);
        onlineList.insert(0, usritem);
        offlineList.remove(ptr);
      }
    }
  for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
    if(ptr->getInfo().uin==uin)
    {
      ptr->setStatus(status);
      if(status==(unsigned long)STATUS_OFFLINE)
      {
        usritem = new UserItem(ptr);
        offlineList.insert(0, usritem);
        onlineList.remove(ptr);
      }
    }
  changeList();
}

void UserList::allOffline()
{
  UserItem *ptr, *usritem;
  for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
  {
    ptr->setStatus((unsigned long)STATUS_OFFLINE);
    usritem = new UserItem(ptr);
    offlineList.insert(0, usritem);
    onlineList.remove(ptr);
  }
  changeList();
}

void UserList::appendItem(UserItem *item)
{
  item->setStatus((unsigned long)STATUS_OFFLINE);
  offlineList.append(item);
}

void UserList::changeList()
{
  UserItem *ptr, *item;
	unsigned long cur_uin = 0;
	unsigned int i;
  setAutoUpdate(FALSE);
	if(currentItem()>=0)
	  cur_uin = getItem(currentItem())->UIN();
  QListBox::clear();
  if(onlineList.count()>0)
  {
    item = new UserItem(tpOnSeparator);
    insertItem(item);
    for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
    {
      item = new UserItem(ptr);
      insertItem(item);
    }
  }
  if(offlineList.count()>0)
  {
    item = new UserItem(tpOffSeparator);
    insertItem(item);
    for(ptr=offlineList.first(); ptr != 0L; ptr=offlineList.next())
    {
      item = new UserItem(ptr);
      insertItem(item);
    }
  }
  if(notInList.count()>0)
  {
    item = new UserItem(tpNILSeparator);
    insertItem(item);
    for(ptr=notInList.first(); ptr != 0L; ptr=notInList.next())
    {
      item = new UserItem(ptr);
      insertItem(item);
    }
  }
  if(cur_uin)
	  for(i=0; i<count(); i++)
      if(getItem(i)->UIN()==cur_uin)
        setCurrentItem(i);
  setAutoUpdate(TRUE);
  update();
}

void UserList::clear()
{
  QListBox::clear();
  onlineList.clear();
  offlineList.clear();
  notInList.clear();
}

const char *UserList::getNickByUIN(unsigned long uin)
{
  UserItem *ptr;
  for(ptr=offlineList.first(); ptr != 0L; ptr=offlineList.next())
    if(ptr->getInfo().uin==uin)
      return ptr->name();
  for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
    if(ptr->getInfo().uin==uin)
      return ptr->name();
  return "Not In List";
}

const char *UserList::getEMailByUIN(unsigned long uin)
{
  UserItem *ptr;
  for(ptr=offlineList.first(); ptr != 0L; ptr=offlineList.next())
    if(ptr->getInfo().uin==uin)
      return ptr->email();
  for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
    if(ptr->getInfo().uin==uin)
      return ptr->email();
  return "Not In List";
}

ContactItem *UserList::getInfoByUIN(unsigned long uin)
{
  UserItem *ptr;
  for(ptr=offlineList.first(); ptr != 0L; ptr=offlineList.next())
    if(ptr->getInfo().uin==uin)
      return ptr->getInfoPtr();
  for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
    if(ptr->getInfo().uin==uin)
      return ptr->getInfoPtr();
  return 0L;
}

void UserList::deleteByUIN(unsigned long uin)
{
  UserItem *ptr;
  for(ptr=offlineList.first(); ptr != 0L; ptr=offlineList.next())
    if(ptr->getInfo().uin==uin)
      offlineList.removeRef(ptr);
  for(ptr=onlineList.first(); ptr != 0L; ptr=onlineList.next())
    if(ptr->getInfo().uin==uin)
      onlineList.removeRef(ptr);
}
