
#define STATUS_OFFLINE  (-1L)
#define STATUS_ONLINE  0x00
#define STATUS_INVISIBLE 0x100
#define STATUS_NA      0x05
#define STATUS_FREE_CHAT 0x20
#define STATUS_OCCUPIED 0x11
#define STATUS_AWAY    0x01
#define STATUS_DND    0x13

typedef struct
{
   unsigned long uin;
   int year;
   int month;
   int day;
   int hour;
   int minute;
   long type;
   long len;
   char* msg;
   char* url;
} CLIENT_MESSAGE, *CLIENT_MESSAGE_PTR;

typedef struct
{
   unsigned long uin;
   char nick[20];
   unsigned long status;
} USER_UPDATE, *USER_UPDATE_PTR;

typedef struct
{
   unsigned long uin;
   char nick[20];
   char first[50];
   char last[50];
   char email[50];
   int auth_required;
} USER_INFO, *USER_INFO_PTR;

typedef struct
{
   unsigned long uin;
   char city[50];
   char country[50];
   int country_code;
   int country_status;
   char state[50];
   char age[15];
   char sex[15];
   char phone[15];
   char url[150];
   char about[1000];
} USER_EXT_INFO, *USER_EXT_INFO_PTR;

typedef struct
{
   unsigned long uin;
   unsigned long status;
   unsigned long last_time; /* last time online or when came online */
   unsigned long current_ip;
   unsigned long port;
   int sok;
   char nick[20];
} Contact_Member, *CONTACT_PTR;

struct result_node
{
   unsigned long uin;
   char nick[20];
   char first[50];
   char last[50];
   char email[50];
   int auth_required;
   struct result_node* next;
};

typedef struct result_node SEARCH_RESULT, *SEARCH_RESULT_PTR;

extern Contact_Member Contacts[ 100 ]; /* no more than 100 contacts max */
extern int Num_Contacts;

extern unsigned long UIN;
extern char passwd[];
extern char server[];
extern unsigned long remote_port;
extern unsigned long set_status;

/* To make dealing with function pointers a LOT easier... */
typedef void (*CALLBACK)();

/* Types of server messages */
enum { EVENT_LOGIN, EVENT_MESSAGE, EVENT_INFO, EVENT_EXT_INFO,
	EVENT_OFFLINE, EVENT_ONLINE, EVENT_STATUS_UPDATE,
	EVENT_SEARCH_RESULTS, EVENT_DISCONNECT, NUM_EVENTS };

/* An array of function pointers to store the callbacks */
void (*event[NUM_EVENTS])(void* data);

/* Assigns "func" to the appropriate "event" pointer */
void ICQ_Register_Callback(int event_type, CALLBACK func);

/* This is the single polling function */
void ICQ_Check_Response(long timeout);

/* Client is responsible for calling this every 2 min or whatever */
void ICQ_Keep_Alive();

int ICQ_Connect();
void ICQ_Disconnect();
void ICQ_Change_Status(int new_status);
void ICQ_Send_Message(long uin, char* text);
void ICQ_Send_URL(long uin, char* url, char* text);
void ICQ_Search(char *email, char *nick, char* first, char* last);
void ICQ_Get_Info(long uin);
void ICQ_Get_Ext_Info(long uin);
void ICQ_Add_User(long uin, char *name);
void ICQ_Rename_User(long uin, char *name);
void ICQ_Delete_User(long uin);
