#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#ifdef _WIN32
  #include <winsock2.h>
#else
  #include <netinet/in.h>
  #include <unistd.h>
  #include <netdb.h>
#endif
#include <time.h>
#include <string.h>

#include "datatype.h"
#include "icq.h"
#include "libicq.h"
#include "receive.h"
#include "send.h"
#include "util.h"

extern BOOL Quit;
extern BOOL Verbose;
extern SOK_T sok;
extern BOOL serv_mess[1024];
extern WORD last_cmd[1024];
extern DWORD our_ip;
extern DWORD our_port;
extern DWORD last_recv_uin;
extern SEARCH_RESULT* Search_Results;
extern BOOL Done_Login;


/******************************************
Handles packets that the server sends to us.
*******************************************/
void Handle_Server_Response()
{
   srv_net_icq_pak pak;
   int s, i, len;

   s = SOCKREAD(sok, &pak.head.ver, sizeof(pak) - 2);
   
   if((serv_mess[Chars_2_Word(pak.head.seq)]) && 
      (Chars_2_Word(pak.head.cmd) != SRV_NEW_UIN))
   {
      if(Chars_2_Word(pak.head.cmd) != SRV_ACK) /* ACKs don't matter */
      {
         if (Verbose)
            fprintf(stdout, "\nIgnored a message cmd  %04x\n", Chars_2_Word(pak.head.cmd));
            
         Send_Ack(Chars_2_Word(pak.head.seq));   /* LAGGGGG!! */
         return;
      }
   }
   
   if(Chars_2_Word(pak.head.cmd) != SRV_ACK)
      serv_mess[Chars_2_Word(pak.head.seq)] = TRUE;
   
   switch(Chars_2_Word(pak.head.cmd))
   {
   case SRV_ACK:
      if(Verbose)
         printf("\nThe server acknowledged the %04x command.\n", 
            last_cmd[Chars_2_Word(pak.head.seq)]);
      break;
      
   case SRV_NEW_UIN:
      printf("\nThe new UIN is %ld!\n", Chars_2_DW(&pak.data[2]));
      break;
      
   case SRV_LOGIN_REPLY:
      Rec_Login(pak);
      break;
      
   case SRV_RECV_MESSAGE:
      Rec_Message(pak);
      break;
      
   case SRV_X1:
      Rec_X1(pak);  
      break;
      
   case SRV_X2:
      Rec_X2(pak);
      break;
      
   case SRV_INFO_REPLY:
      Rec_Info(pak);
      break;
      
   case SRV_EXT_INFO_REPLY:
      Rec_ExtInfo(pak);  
      break;
      
   case SRV_USER_OFFLINE:  
      Rec_UserOffline(pak);
      break;
      
   case SRV_USER_ONLINE:
      Rec_UserOnline(pak);
      break;
      
   case SRV_STATUS_UPDATE:
      Rec_StatusUpdate(pak);  
      break;
      
   case SRV_TRY_AGAIN:
      Rec_TryAgain(pak);  
      break;
      
   case SRV_GO_AWAY:
      Rec_GoAway(pak);  
      break;
      
   case SRV_USER_FOUND:
      Rec_UserFound(pak);  
      break;
      
   case SRV_END_OF_SEARCH:
      Rec_EndOfSearch(pak);  
      break;
      
   case SRV_SYS_DELIVERED_MESS:
      Rec_SysDeliveredMess(pak); 
      break;

   default: /* commands we dont handle yet */
       printf( CLIENTCOL "\nThe response was %04X\t", Chars_2_Word( pak.head.cmd ) );
       printf( "The version was %X\t", Chars_2_Word( pak.head.ver ) );

       printf( "\nThe SEQ was %04X\t", Chars_2_Word( pak.head.seq ) );
       len = s - ( sizeof( pak.head ) - 2 );
       printf( "The size was %d\n", len );
       if ( Verbose )
       {
          for ( i=0; len > i; i++ )
          {
             printf( "%02X ",( unsigned char ) pak.data[i] );
             if ( i % 10 == 9 )
             {
                printf( "\n" );
             }
          }
       }
       printf( NOCOL "\n" );

       Send_Ack(Chars_2_Word( pak.head.seq ) ); /* fake like we know what we're doing*/
       break;
   }
}


void Rec_Login(srv_net_icq_pak pak)
{
   Send_Ack(Chars_2_Word(pak.head.seq));

   UIN = Chars_2_DW(&pak.data[0]);
   our_ip = Chars_2_DW(&pak.data[4]);

   if(Verbose)
   {
      printf("\nLogin successful! UIN : %lu\n", UIN);
      printf(" IP : %u.%u.%u.%u\n", pak.data[4], pak.data[5], pak.data[6], pak.data[7]);
   }

   Send_FinishLogin();
   Send_ContactList();
   Send_ChangeStatus(set_status);

   if(event[EVENT_LOGIN] != NULL)
     (*event[EVENT_LOGIN])(NULL);
}


void Rec_X1(srv_net_icq_pak pak)
{
   if(Verbose) printf("\nAcknowleged SRV_X1 0x021C Begin messages?\n");
   Send_Ack(Chars_2_Word(pak.head.seq));
}


void Rec_X2(srv_net_icq_pak pak)
{
   Send_Ack(Chars_2_Word(pak.head.seq));
   if(Verbose) printf("\nAcknowleged SRV_X2 0x00E6 Done old messages?\n");
   Done_Login = TRUE;
   Send_GotMessages();
}


void Rec_Message(srv_net_icq_pak pak)
{
   RECV_MESSAGE_PTR r_mesg;
   CLIENT_MESSAGE c_mesg;
   char* data, *tmp;

   Send_Ack(Chars_2_Word(pak.head.seq));

   r_mesg = (RECV_MESSAGE_PTR)pak.data;
   c_mesg.uin = Chars_2_DW(r_mesg->uin);
   c_mesg.month = (int)(r_mesg->month);
   c_mesg.day = (int)(r_mesg->day);
   c_mesg.year = Chars_2_Word(r_mesg->year);
   c_mesg.hour = (int)(r_mesg->hour);
   c_mesg.minute = (int)(r_mesg->minute);
   c_mesg.type = Chars_2_Word(r_mesg->type);
   c_mesg.len = Chars_2_Word(r_mesg->len);

   data = (char*)(r_mesg->len + 2);
   
   if(c_mesg.type == URL_MESS)
   {
      tmp = strchr(data, '\xFE');
      if(!tmp) return;
      *tmp = 0;
      c_mesg.msg = data;
      
      data = ++tmp;
      c_mesg.url = data;
   }
   else
   {
      c_mesg.msg = data;
   }

   last_recv_uin = Chars_2_DW(r_mesg->uin);
   
   if(event[EVENT_MESSAGE] != NULL)
      (*event[EVENT_MESSAGE])(&c_mesg);
}


void Rec_SysDeliveredMess(srv_net_icq_pak pak)
{
   SIMPLE_MESSAGE_PTR s_mesg;
   CLIENT_MESSAGE c_mesg;
   char* data, *tmp;
   
   Send_Ack(Chars_2_Word(pak.head.seq));

   s_mesg = (SIMPLE_MESSAGE_PTR)pak.data;
   last_recv_uin = Chars_2_DW(s_mesg->uin);
      
   s_mesg = (SIMPLE_MESSAGE_PTR)pak.data;
   c_mesg.uin = Chars_2_DW(s_mesg->uin);
   c_mesg.month = 0;
   c_mesg.day = 0;
   c_mesg.year = 0;       /*  will fill these in later :) */
   c_mesg.hour = 0;
   c_mesg.minute = 0;
   c_mesg.type = Chars_2_Word(s_mesg->type);
   c_mesg.len = Chars_2_Word(s_mesg->len);

   data = (char*)(s_mesg->len + 2);
   
   if(c_mesg.type == URL_MESS)
   {
      tmp = strchr(data, '\xFE');
      if(!tmp) return;
      *tmp = 0;
      c_mesg.msg = data;
      
      data = ++tmp;
      c_mesg.url = data;
   }
   else
   {
      c_mesg.msg = data;
   }

   if(event[EVENT_MESSAGE] != NULL)
     (*event[EVENT_MESSAGE])(&c_mesg);
}


void Rec_Info(srv_net_icq_pak pak)
{
   USER_INFO info;
   char *tmp;
   int len;

   Send_Ack(Chars_2_Word(pak.head.seq));

   info.uin = Chars_2_DW(&pak.data[2]);
   strncpy(info.nick, &pak.data[8], 20);
   
   len = Chars_2_Word( &pak.data[6] );
   tmp = &pak.data[8 + len ];
   strncpy(info.first, (char*)(tmp + 2), 50);

   len = Chars_2_Word( tmp );
   tmp += len + 2;
   strncpy(info.last, (char*)(tmp + 2), 50);
   
   len = Chars_2_Word( tmp );
   tmp += len + 2;
   strncpy(info.email, (char*)(tmp + 2), 50);
   
   len = Chars_2_Word( tmp );
   tmp += len + 2;
   info.auth_required = *tmp;

   if(event[EVENT_INFO] != NULL) 
     (*event[EVENT_INFO])(&info);
}


void Rec_ExtInfo(srv_net_icq_pak pak)
{
   USER_EXT_INFO extinfo;
   unsigned char *tmp;
   int len;

   Send_Ack(Chars_2_Word(pak.head.seq));

   extinfo.uin = Chars_2_DW(&pak.data[2]);
   strncpy(extinfo.city, &pak.data[8], 50);

   len = Chars_2_Word( &pak.data[6] );
   extinfo.country_code = Chars_2_Word(&pak.data[8 + len]);
   extinfo.country_status = pak.data[len + 10];

   if(Get_Country_Name(extinfo.country_code) != NULL)
      strncpy(extinfo.country, Get_Country_Name(extinfo.country_code), 50);
   
   tmp = &pak.data[11 + len];
   strncpy(extinfo.state, (char*)(tmp + 2), 50);

   len = Chars_2_Word(tmp);
   if(Chars_2_Word(tmp + len + 2) != 0xffff)
      sprintf(extinfo.age, "%d", Chars_2_Word(tmp + len + 2));
   else
      strcpy(extinfo.age, "Not Entered");
      
   if(*(tmp + len + 4) == 2)
      strcpy(extinfo.sex, "Male");
   else if(*(tmp + len + 4) == 1)
      strcpy(extinfo.sex, "Female");
   else
      strcpy(extinfo.sex, "Not Specified");
      
   tmp += len + 5;
   strncpy(extinfo.phone, (char*)(tmp + 2), 15);

   len = Chars_2_Word( tmp );
   tmp += len + 2;
   strncpy(extinfo.url, (char*)(tmp + 2), 150);   
   
   len = Chars_2_Word( tmp );
   tmp += len + 2;
   strncpy(extinfo.about, (char*)(tmp + 2), 1000);

   if(event[EVENT_EXT_INFO] != NULL) 
     (*event[EVENT_EXT_INFO])(&extinfo);
}


void Rec_UserOffline(srv_net_icq_pak pak)
{
   USER_UPDATE user_update;
   int index;

   Send_Ack(Chars_2_Word(pak.head.seq));
   
   user_update.uin = pak.data[3];
   user_update.uin <<=8;
   user_update.uin += pak.data[2];
   user_update.uin <<=8;
   user_update.uin += pak.data[1];
   user_update.uin <<=8;
   user_update.uin += pak.data[0];

   for(index = 0; index < Num_Contacts; index++)
     if(Contacts[index].uin == user_update.uin) break;

   if(index <= Num_Contacts)
   {
      Contacts[index].status = STATUS_OFFLINE;
      Contacts[index].last_time = time(NULL);
   }

   strcpy(user_update.nick, uin2nick(user_update.uin));
   user_update.status = STATUS_OFFLINE;

   if(event[EVENT_OFFLINE] != NULL)
        (*event[EVENT_OFFLINE])(&user_update);
}


void Rec_UserOnline(srv_net_icq_pak pak)
{
   USER_UPDATE user_update;
   int index;

   Send_Ack(Chars_2_Word(pak.head.seq));

   user_update.uin = Chars_2_DW(&pak.data[0]);
   user_update.status = Chars_2_DW(&pak.data[17]);
   strcpy(user_update.nick, uin2nick(user_update.uin));
      
   for(index = 0; index < Num_Contacts; index++)
     if(Contacts[index].uin == user_update.uin) break;

   if(index <= Num_Contacts)
   {
      Contacts[index].status = user_update.status;
      Contacts[index].current_ip = ntohl(Chars_2_DW(&pak.data[4]));
      Contacts[index].port = Chars_2_DW(&pak.data[8]);
      Contacts[index].last_time = time(NULL);
   }
  
   if(event[EVENT_ONLINE] != NULL)
     (*event[EVENT_ONLINE])(&user_update); 
}


void Rec_StatusUpdate(srv_net_icq_pak pak)
{
   USER_UPDATE user_update;
   int index;

   Send_Ack(Chars_2_Word(pak.head.seq));
   
   user_update.uin = pak.data[3];
   user_update.uin <<= 8;
   user_update.uin += pak.data[2];
   user_update.uin <<= 8;
   user_update.uin += pak.data[1];
   user_update.uin <<= 8;
   user_update.uin += pak.data[0];

   user_update.status = pak.data[7];
   user_update.status <<= 8;
   user_update.status += pak.data[6];
   user_update.status <<= 8;
   user_update.status += pak.data[5];
   user_update.status <<= 8;
   user_update.status += pak.data[4];
   
   for(index = 0; index < Num_Contacts; index++)
     if(Contacts[index].uin == user_update.uin) break;
   if(index <= Num_Contacts) Contacts[index].status = user_update.status;
   
   strcpy(user_update.nick, uin2nick(user_update.uin));

   if(event[EVENT_STATUS_UPDATE] != NULL)
     (*event[EVENT_STATUS_UPDATE])(&user_update);

}


void Rec_TryAgain(srv_net_icq_pak pak)
{
   int i;
   printf( MESSCOL "\nServer is busy please try again.\nTrying again...\n");
   Send_BeginLogin(UIN, &passwd[0], our_ip, our_port);
   for ( i = 0; i < 1024; i++ )
   {
      serv_mess[ i ]=FALSE;
   }
}


void Rec_GoAway(srv_net_icq_pak pak)
{
   printf("\nServer has forced us to disconnect.  This may be because of a bad password.\n" );
   exit(0);
}


void Rec_UserFound(srv_net_icq_pak pak)
{
   SEARCH_RESULT* current_search_result, *prev_search_result;
   char* tmp;
   int len;
   
   current_search_result = Search_Results;
   prev_search_result = NULL;
   while(current_search_result != NULL)
   {
      prev_search_result = current_search_result;
      current_search_result = current_search_result->next;
   }
      
   current_search_result = malloc(sizeof(SEARCH_RESULT));
   if(prev_search_result != NULL)
      prev_search_result->next = current_search_result;
   current_search_result->next = NULL;
   current_search_result->uin = Chars_2_DW(&pak.data[2]);
   len = Chars_2_Word(&pak.data[6]);
   strncpy(current_search_result->nick, (char*)(&pak.data[8]), 20);
   tmp = &pak.data[len + 8];
   len = Chars_2_Word(tmp);
   strncpy(current_search_result->first, (char*)(tmp + 2), 50);
   tmp += len + 2;
   len = Chars_2_Word(tmp);
   strncpy(current_search_result->last, (char*)(tmp + 2), 50);
   tmp += len + 2;
   len = Chars_2_Word(tmp);
   strncpy(current_search_result->email, (char*)(tmp + 2), 50);
   tmp += len + 2;
   current_search_result->auth_required = *tmp;
      
   if(Search_Results == NULL) Search_Results = current_search_result;
}


void Rec_EndOfSearch(srv_net_icq_pak pak)
{
   SEARCH_RESULT* current_search_result;
   
   Send_Ack(Chars_2_Word( pak.head.seq ) );

   if(Verbose) printf("Search Results:\n");

   current_search_result = Search_Results;
   while(current_search_result != NULL)
   {
      if(Verbose)
         printf("  UIN: %ld\tNick: %s\tName: %s %s\tE-mail: %s\n",
            current_search_result->uin, current_search_result->nick,
            current_search_result->first, current_search_result->last,
            current_search_result->email);
            
      current_search_result = current_search_result->next;
   }

   if(event[EVENT_SEARCH_RESULTS] != NULL)
     (*event[EVENT_SEARCH_RESULTS])((void*)Search_Results);
}





