/*
   irxevent  - infra-red xevent sender

   Heinrich Langos  <heinrich@null.net>
   small modifications by Christoph Bartelmus (columbus@hit.handshake.de)

   irxevent is based on irexec(Copyright (C) 1998 Trent Piepho)
   and irx.c(no copyright notice found)

   irxevent is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published
   by the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   irxevent is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */
/*

CHANGES:
-Version 0.1 
      -Initial Release

-Version 0.2 
      -no more XWarpPointer... sending Buttonclicks to off-screen
       applications works becaus i also fake the EnterNotify and LeaveNotify
      -support for keysymbols rather than characters... so you can use
       Up or Insert or Control_L ... maybe you could play xquake than :*)

-Version 0.3
      -bugfix for looking for subwindows of non existing windows 
      -finaly a README file

for more information see the irxevent.README file

*/


#include <errno.h>
#include <unistd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <time.h>
#include <unistd.h>

#include "lirc_client.h"

char *progname;

Display *dpy;
Window root;
XEvent xev;
Window w;

Window find_window(Window top,char *name)
{
	char *wname,*iname;
	XClassHint xch;
	Window *children,foo;
	unsigned int nc;

	/* First the base case */
	if (XFetchName(dpy,top,&wname)){
		if (!strncmp(wname,name,strlen(name)))  {
			XFree(wname);
			/*printf("found it by wname %x \n",top);*/
			return(top);  /* found it! */
		};
		XFree(wname);
	};

	if(XGetIconName(dpy,top,&iname)){
		if (!strncmp(iname,name,strlen(name)))  {
			XFree(iname);
			/*printf("found it by iname %x \n",top);*/
			return(top);  /* found it! */
		};
		XFree(iname);
	};

	if(XGetClassHint(dpy,top,&xch))  {
		if(!strcmp(xch.res_class,name))  {
			XFree(xch.res_name); XFree(xch.res_class);
			/*printf("res_class '%s' res_name '%s' %x \n",
			  xch.res_class,xch.res_name,top);*/
			return(top);  /* found it! */
		};
		XFree(xch.res_name); XFree(xch.res_class);
	};

	if(!XQueryTree(dpy,top,&foo,&foo,&children,&nc) || children==NULL) {
		return(0);  /* no more windows here */
	};

	/* check all the sub windows */
	for(;nc>0;nc--)  {
		top = find_window(children[nc-1],name);
		if(top) break;  /* we found it somewhere */
	};
	if(children!=NULL) XFree(children);
	return(top);
}

Window find_sub_window(Window top,char *name,int *x, int *y)
{
  Window base;
  Window *children,foo,target=0;
  unsigned int nc,
    rel_x,rel_y,width,height,border,depth,
    new_x=1,new_y=1,
    targetsize=1000000;

  base=find_window(top, name);
  if (!base) {return base;};
  if(!XQueryTree(dpy,base,&foo,&foo,&children,&nc) || children==NULL) {
    return(base);  /* no more windows here */
  };

  /* check if we hit a sub window and find the smallest one */
  for(;nc>0;nc--)  {
    if(XGetGeometry(dpy, children[nc-1], &foo, &rel_x, &rel_y, 
		    &width, &height, &border, &depth)){
      if ((rel_x<=*x)&&(*x<=rel_x+width)&&(rel_y<=*y)&&(*y<=rel_y+height)){
	/*printf("found a subwindow %x +%d +%d  %d x %d   \n",children[nc-1],
	  rel_x,rel_y,width,height);*/
	if ((width*height)<targetsize){
	  target=children[nc-1];
	  targetsize=width*height;
	  new_x=*x-rel_x;
	  new_y=*y-rel_y;
	  /*bull's eye ...*/
	}
      }	
    }
  };
  if(children!=NULL) XFree(children);
  if (target){
    *x=new_x;
    *y=new_y;
    return target;
  }else
    return base;
}


void make_button(int button,int x,int y,XButtonEvent *xev)
{
	xev->type = ButtonPress;
	xev->display=dpy;
	xev->root=root;
	xev->subwindow=None;
	xev->time=CurrentTime;
	xev->x=x;xev->y=y;
	xev->x_root=1; xev->y_root=1;
	xev->state=0;
	xev->button=button;
	xev->same_screen=True;

	return;
}

void make_key(char *keyname,XKeyEvent *xev)
{
	xev->type = KeyPress;
	xev->display=dpy;
	xev->root=root;
	xev->subwindow = None;
	xev->time=CurrentTime;
	xev->x=1; xev->y=1;
	xev->x_root=1; xev->y_root=1;
	xev->state=0;
	xev->same_screen = True;

	xev->keycode=XKeysymToKeycode(dpy,XStringToKeysym(keyname));
       	/*printf("keycode is %x\n",xev->keycode);*/
	return ;
}

void sendfocus(Window w,int in_out)
{
  XFocusChangeEvent focev;

  focev.display=dpy;
  focev.type=in_out;
  focev.window=w;
  focev.mode=NotifyNormal;
  focev.detail=NotifyPointer;
  XSendEvent(dpy,w,True,FocusChangeMask,(XEvent*)&focev);
  XSync(dpy,True);

  return;
}

void sendpointer(Window w,int in_out)
{
  XCrossingEvent crossev;
  crossev.type=in_out;
  crossev.display=dpy;
  crossev.window=w;
  crossev.root=root;
  crossev.subwindow=None;
  crossev.time=CurrentTime;
  crossev.x=1;
  crossev.y=1;
  crossev.x_root=1;
  crossev.y_root=1;
  crossev.mode=NotifyNormal;
  crossev.detail=NotifyNonlinear;
  crossev.same_screen=True;
  crossev.focus=True;
  crossev.state=0;
  XSendEvent(dpy,w,True,EnterWindowMask|LeaveWindowMask,(XEvent*)&crossev);
  XSync(dpy,True);
  return;
}

void sendkey(char *keyname,Window w)
{
  make_key(keyname ,(XKeyEvent*)&xev);
  xev.xkey.window=w;
  /*  sendfocus(w,FocusIn);*/
  XSendEvent(dpy,w,True,KeyPressMask,&xev);
  xev.type = KeyRelease;
  xev.xkey.time += 200;
  usleep(200000);
  XSendEvent(dpy,w,True,KeyReleaseMask,&xev);
  XSync(dpy,True);
  /*  sendfocus(w,FocusOut);*/
  return;
}

void sendbutton(int button, int x, int y, Window w)
{
  make_button(button,x,y,(XButtonEvent*)&xev);
  xev.xbutton.window=w;
  sendpointer(w,EnterNotify);
    /*XWarpPointer(dpy, None, w, 0,0,0,0, x,y);*/

  XSendEvent(dpy,w,True,ButtonPressMask,&xev);
  XSync(dpy,True);
  xev.type = ButtonRelease;
  xev.xkey.time += CurrentTime;
  xev.xkey.state|=0x100;
  usleep(100000);
  XSendEvent(dpy,w,True,ButtonReleaseMask,&xev);
  sendpointer(w,LeaveNotify);
  XSync(dpy,True);

  return;
}

void die(char *fmt,...)
{
  va_list args;

  va_start(args, fmt);
  vfprintf(stderr, fmt, args);
  exit(1);
}

int check(char *s)
{
	int d;
	char *buffer;

	buffer=malloc(strlen(s));
	if(buffer==NULL)
	{
		fprintf(stderr,"%s: out of memory\n",progname);
		return(-1);
	}

	if(2!=sscanf(s,"Key %s %s\n",buffer,buffer) &&
	   4!=sscanf(s,"Button %d %d %d %s\n",&d,&d,&d,buffer))
	{
		fprintf(stderr,"%s: bad config string \"%s\"\n",progname,s);
		return(-1);
	}
	return(0);
}

int main(int argc, char *argv[])
{
  char keyname[64];
  int pointer_button,pointer_x,pointer_y;
  char windowname[64];
  struct lirc_config *config;

  progname=argv[0];
  if(argc>2)  {
    fprintf(stderr,"Usage: %s <config file>\n",argv[0]);
    exit(1);
  };

  dpy=XOpenDisplay(NULL);
  if(dpy==NULL) die("Can't open DISPLAY.\n");
  root=RootWindow(dpy,DefaultScreen(dpy));

  if(lirc_init("irxevent")==-1) exit(EXIT_FAILURE);

  if(lirc_readconfig(argc==2 ? argv[1]:NULL,&config,check)==0)
  {
	  char *ir;
	  char *c;
	  
	  while((ir=lirc_nextir())!=NULL)
	  {
		  while((c=lirc_ir2char(config,ir))!=NULL)
		  {
#ifdef DEBUG			  
			  printf("Recieved code: %sSending event: ",ir);
#endif
			  if(2==sscanf(c,"Key %s %s\n",keyname,windowname))
			  {
				  if((w=find_window(root,windowname)))
				  {
					  sendkey(keyname,w);
#ifdef DEBUG
					  printf("keyname:%s  windowname:%s\n",keyname,windowname);
#endif
				  }
				  else
				  {
#ifdef DEBUG
					  printf("target window '%s' not found \n",windowname);
#endif
				  }
			  }
			  else if(4==sscanf(c,"Button %d %d %d %s\n",
					    &pointer_button,&pointer_x,
					    &pointer_y,windowname))
			  {
				  
				  if((w=find_sub_window(root,windowname,&pointer_x,&pointer_y)))
				  {
					  sendbutton(pointer_button,pointer_x,pointer_y,w);
#ifdef DEBUG
					  printf(" %s\n",c);
#endif
				  }
				  else
				  {
#ifdef DEBUG
					  printf("target window '%s' not found \n",windowname);
#endif
				  }
			  }
		  }
		  free(ir);
	  }
	  lirc_freeconfig(config);
  }

  lirc_deinit();

  exit(0);
}
