/*
  
  record -  application for recording IR-codes for usage with lirc
  
  Copyright (C) 1998 Christoph Bartelmus (columbus@hit.handshake.de)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/un.h>
#include <errno.h>

#include "../"DAEMON"/dump_config.h"
#include "../"DAEMON"/ir_remote.h"

#define VERSION "0.2"
#define BUTTON 80+1

/* the longest signal I've seen up to now was 48-bit signal with header */

#define MAX_SIGNALS 200

struct ir_remote remote;
struct ir_ncode code;

int main(int argc,char **argv)
{
	fd_set fds;
	struct timeval timeout;
	char *progname,*filename;
	FILE *fout;
	int lirc;
	int ret;
	unsigned long data,average,repeat[3],signals[MAX_SIGNALS+1];
	int count,mode;

	progname=argv[0];

	while(1)
	{
		int c;
		static struct option long_options[] =
		{
			{"help",no_argument,NULL,'h'},
			{"version",no_argument,NULL,'v'},
			{0, 0, 0, 0}
		};
		c = getopt_long(argc,argv,"hv",long_options,NULL);
		if(c==-1)
			break;
		switch (c)
		{
		case 'h':
			printf("Usage: %s file\n",progname);
			printf("\t -h --help\t\tdisplay this message\n");
			printf("\t -v --version\t\tdisplay version\n");
			return(0);
		case 'v':
			printf("record %s\n",VERSION);
			return(EXIT_SUCCESS);
		default:
			return(EXIT_FAILURE);
		}
	}
	if(optind+1!=argc)
	{
		fprintf(stderr,"%s: invalid argument count\n",progname);
		return(EXIT_FAILURE);
	}
	filename=argv[optind];
	fout=fopen(filename,"w");
	if(fout==NULL)
	{
		fprintf(stderr,"%s: could not open file %s\n",progname,
			filename);
		perror(progname);
		return(EXIT_FAILURE);
	}
	printf("record -  application for recording IR-codes"
	       " for usage with lirc\n"
	       "\n"  
	       "Copyright (C) 1998 Christoph Bartelmus"
	       "(columbus@hit.handshake.de)\n");
	printf("\n\n");
	
	lirc=open("/dev/lirc",O_RDONLY|O_NONBLOCK);
	if(lirc==-1)
	{
		fprintf(stderr,"%s: could not open /dev/lirc\n",progname);
		perror(progname);
		fclose(fout);
		return(EXIT_FAILURE);
	}

	printf("This program will record the signals from your remote control\n"
	       "and create a config file for lircd\n\n");

	remote.flags=RAW_CODES;
	remote.eps=20;                 /* should work with all remotes */
	remote.name=filename;

	while(read(lirc,&data,sizeof(unsigned long))==sizeof(unsigned long));

	printf("First, please hold down an arbitrary button\n");

	mode=count=0;
	average=0;
	while(1)
	{
		FD_ZERO(&fds);
		FD_SET(lirc,&fds);
		/* timeout after 10 secs */
		timeout.tv_sec=10;
		timeout.tv_usec=0;
		ret=select(lirc+1,&fds,NULL,NULL,&timeout);
		if(ret==-1)
		{
			fprintf(stderr,"%s: select() failed\n",progname);
			perror(progname);
			close(lirc);
			fclose(fout);
			exit(EXIT_FAILURE);
		}
		else if(ret==0)
		{
			fprintf(stderr,"%s: no data for 10 secs, aborting\n",progname);
			close(lirc);
			fclose(fout);
			exit(EXIT_FAILURE);
		}

		ret=read(lirc,&data,sizeof(unsigned long));
		if(ret!=sizeof(unsigned long))
		{
			fprintf(stderr,"%s: read() failed\n",progname);
			perror(progname);
			close(lirc);
			fclose(fout);
			exit(EXIT_FAILURE);
		}
		if(is_space(data)) /* currently I am only interested in spaces */
		{
			if(mode==0)
			{
				if(average==0)
				{
					if(data>100000) continue;
					average=data;
				}
				else
				{
					if(data>10*average ||
					   /* this MUST be a gap */
					   (count>10 && data>5*average))  
						/* this should be a gap */
					{
						remote.gap=data;
						mode=1;
					}
					average=(average*count+data)/(count+1);
				}
				if(mode==1)
				{
					average=0;
					count=0;
					printf("\nFound gap: %ld\n",remote.gap);
					printf("Please wait a moment...\n");
					sleep(5);
					while(read(lirc,&data,sizeof(unsigned long))
					      ==sizeof(unsigned long));
					printf("\nNow, please hold down an"
					       " arbitrary button again to"
					       " confirm this value.\n");
					continue;
				}
			}
			else if(mode==1)
			{
				if(count>MAX_SIGNALS)
				{
					printf("Failed to confirm value.\n");
					printf("Please try again.\n");
					close(lirc);
					fclose(fout);
					exit(EXIT_FAILURE);
				}
				if(average==0)
				{
					if(data>100000) continue;
					average=data;
				}
				else
				{
					
					if(data>10*average ||
					   /* this MUST be a gap */
					   (count>10 && data>5*average))  
						/* this should be a gap */
					{
						if(expect(&remote,data,remote.gap))
						{
							/* Succeded to confirm value */
							count=0;
							mode=2;
							continue;
						}
					}
					average=(average*count+data)/(count+1);
				}
			}
		}
		if(mode==2)
		{
			if(count<3)
			{
				repeat[count]=data;
			}
			else
			{
				if(expect(&remote,data,remote.gap))
				{
					printf("Found repeat code: %ld %ld %ld\n",
					       repeat[0],repeat[1],repeat[2]);
					remote.prepeat=repeat[0];
					remote.srepeat=repeat[1];
					remote.ptrail=repeat[2];
				}
				break;
			}
		}
		count++;
	}
	printf("\nSucceded to confirm value.\n");
	printf("Now enter the names for the buttons.\n");

	fprint_remote_head(fout,&remote);
	fprint_remote_signal_head(fout,&remote);
	
	while(1)
	{
		char buffer[BUTTON];
		char *string;

		printf("\nPlease enter the name for the next button\n");
		string=fgets(buffer,BUTTON,stdin);

		if(string!=buffer)
		{
			fprintf(stderr,"%s: fgets() failed\n",progname);
			close(lirc);
			fclose(fout);
			return(EXIT_FAILURE);
		}
		buffer[strlen(buffer)-1]=0;
		if(strchr(buffer,' ') || strchr(buffer,'\t'))
		{
			printf("The name may not contain any whitespace.\n");
			printf("Please try again.\n");
			continue;
		}
		if(strlen(buffer)==0)
		{
			break;
		}
		
		while(read(lirc,&data,sizeof(unsigned long))
		      ==sizeof(unsigned long));
		printf("\nNow press button \"%s\".\n",buffer);

		count=0;
		while(count<MAX_SIGNALS)
		{
			FD_ZERO(&fds);
			FD_SET(lirc,&fds);
			/* timeout after 10 secs */
			timeout.tv_sec=10;
			timeout.tv_usec=0;
			ret=select(lirc+1,&fds,NULL,NULL,&timeout);
			if(ret==-1)
			{
				fprintf(stderr,"%s: select() failed\n",progname);
				perror(progname);
				close(lirc);
				fclose(fout);
				exit(EXIT_FAILURE);
			}
			if(ret==0)
			{
				if(count==0)
				{
					printf("%s: no data for 10 secs, aborting\n",progname);
					fprint_remote_signal_foot(fout,&remote);
					fprint_remote_foot(fout,&remote);
					
					close(lirc);
					fclose(fout);
					return(EXIT_SUCCESS);
				}
				data=remote.gap;
			}
			else
			{
				ret=read(lirc,&data,sizeof(unsigned long));
				if(ret!=sizeof(unsigned long))
				{
					fprintf(stderr,"%s: read() failed\n",progname);
					perror(progname);

					fprint_remote_signal_foot(fout,&remote);
					fprint_remote_foot(fout,&remote);
					close(lirc);
					fclose(fout);
					exit(EXIT_FAILURE);
				}
			}

			if(count==0)
			{
				if(!is_space(data) ||
				   data<remote.gap-remote.gap*remote.eps/100)
				{
					printf("Sorry, something went wrong.\n");
					printf("Try again.\n");
					break;
				}
			}
			else
			{
				if(is_space(data) && data>remote.gap-remote.gap*remote.eps/100)
				{
					printf("Got it.\n");
					printf("Signal length is %d\n",count-1);
					if(count%2)
					{
						printf("That's weired because the signal length must be odd!\n");
						printf("Try again.\n");
						count=0;
						continue;
					}
					else
					{
						code.name=buffer;
						code.length=count-1;
						code.signals=signals;
						fprint_remote_signal(fout,&remote,&code);
						break;
					}
				}
				signals[count-1]=data&(PULSE_BIT-1);
			}
			count++;
		}
		if(count==MAX_SIGNALS)
		{
			printf("Signal is too long.\n");
		}
	}
	fprint_remote_signal_foot(fout,&remote);
	fprint_remote_foot(fout,&remote);

	close(lirc);
	fclose(fout);
	return(EXIT_SUCCESS);
}

