/*
  dump_config.c - dumps data structures into file

  Copyright (C) 1998 Pablo d'Angelo (pablo@ag-trek.allgaeu.org)
  
  This is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  It is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "dump_config.h"
#include "config.h"

void fprint_flags(FILE *f, int flags)
{
	int i;
	int begin=0;

	for(i=0;i<NR_FLAGS;i++)
	{
		if(flags&all_flags[i].flag)
		{
			if(begin==0) fprintf(f, "  flags ");
			else if(begin==1) fprintf(f,"|");
			fprintf(f,"%s",all_flags[i].name);
			begin=1;
		}
	}
	if(begin==1) fprintf(f,"\n");
}

void fprint_remotes(FILE *f, struct ir_remote *all){

    	while(all)
	{
                fprint_remote(f, all);
                fprintf(f, "\n\n");
                all=all->next;
        }
}

void fprint_remote_head(FILE *f, struct ir_remote *rem)
{
	fprintf(f, "begin remote\n\n");
	if(!is_raw(rem)){
		fprintf(f, "  name  %s\n",rem->name);
		fprintf(f, "  bits        %5d\n",rem->bits);
		fprint_flags(f,rem->flags);
		fprintf(f, "  eps         %5d\n",rem->eps);
		fprintf(f, "  aeps        %5d\n\n",rem->aeps);
		fprintf(f, "  header      %5d %5d\n",rem->phead, rem->shead);
		fprintf(f, "  one         %5d %5d\n",rem->pone, rem->sone);
		fprintf(f, "  zero        %5d %5d\n",rem->pzero, rem->szero);
		fprintf(f, "  ptrail      %5d\n",rem->ptrail);
		fprintf(f, "  plead       %5d\n",rem->plead);
		fprintf(f, "  foot        %5d %5d\n",rem->pfoot, rem->sfoot);
		fprintf(f, "  repeat      %5d %5d\n",rem->prepeat, rem->srepeat);
                fprintf(f, "  pre_data_bits   %d\n",rem->pre_data_bits);
#               ifdef LONG_IR_CODE
		fprintf(f, "  pre_data       0x%llX\n",rem->pre_data);
#               else
		fprintf(f, "  pre_data       0x%lX\n",rem->pre_data);
#               endif
		fprintf(f, "  post_data_bits  %d\n",rem->post_data_bits);
#               ifdef LONG_IR_CODE
		fprintf(f, "  post_data      0x%llX\n",rem->post_data);
#               else
		fprintf(f, "  post_data      0x%lX\n",rem->post_data);
#               endif
		fprintf(f, "  pre         %5d %5d\n",rem->pre_p, rem->pre_s);
		fprintf(f, "  post        %5d %5d\n",rem->post_p, rem->post_s);
		fprintf(f, "  gap          %lu\n",rem->gap);
		fprintf(f, "  repeat_bit      %d\n\n",rem->repeat_bit);
	}
	else
	{
		fprintf(f, "  name   %s\n",rem->name);
		fprint_flags(f,rem->flags);
		fprintf(f, "  eps         %5d\n",rem->eps);
		fprintf(f, "  aeps        %5d\n\n",rem->aeps);
		fprintf(f, "  ptrail      %5d\n",rem->ptrail);
		fprintf(f, "  repeat %5d %5d\n",rem->prepeat, rem->srepeat);
		fprintf(f, "  gap    %lu\n\n",rem->gap);
	}
}

void fprint_remote_foot(FILE *f, struct ir_remote *rem)
{
	fprintf(f, "end remote\n");
}

void fprint_remote_signal_head(FILE *f, struct ir_remote *rem)
{
	if(!is_raw(rem))
		fprintf(f, "      begin codes\n");
	else
		fprintf(f, "      begin raw_codes\n\n");
}

void fprint_remote_signal_foot(FILE *f, struct ir_remote *rem)
{
	if(!is_raw(rem))
		fprintf(f, "      end codes\n\n");
	else
		fprintf(f, "      end raw_codes\n\n");
}

void fprint_remote_signal(FILE *f,struct ir_remote *rem, struct ir_ncode *codes)
{
	int i,j;

	if(!is_raw(rem))
	{
#               ifdef LONG_IR_CODE
		fprintf(f, "          %-24s 0x%016llX\n",codes->name, codes->code);
#               else
		fprintf(f, "          %-24s 0x%016lX\n",codes->name, codes->code);
#               endif
	}
	else
	{
		fprintf(f, "          name %s\n",codes->name);
		j=0;
		for(i=0;i<codes->length;i++){
			if (j==0){
				fprintf(f, "          %7lu",codes->signals[i]);
			}else if (j<5){
				fprintf(f, " %7lu",codes->signals[i]);
			}else{
				fprintf(f, " %7lu\n",codes->signals[i]);
				j=-1;
			}
			j++;
		}
		codes++;
		if (j==0)
		{
			fprintf(f,"\n");
		}else
		{
			fprintf(f,"\n\n");
			j=0;
		}
	}
}

void fprint_remote_signals(FILE *f, struct ir_remote *rem)
{
        struct ir_ncode *codes;
	
	fprint_remote_signal_head(f,rem);
	codes=rem->codes;
	while(codes->name!=NULL)
	{
		fprint_remote_signal(f,rem,codes);
		codes++;
	}
	fprint_remote_signal_foot(f,rem);
}


void fprint_remote(FILE *f, struct ir_remote *rem)
{	
	fprint_remote_head(f,rem);
	fprint_remote_signals(f,rem);
	fprint_remote_foot(f,rem);
}
