/*

  lircmd - Linux Infra-red Remote Control Mouse Daemon
  
  Copyright (C) 1998 Christoph Bartelmus (columbus@hit.handshake.de)
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <string.h>
#include <syslog.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>

#define CLICK_DELAY 50000 /* usecs */
#define PACKET_SIZE 256
#define WHITE_SPACE " \t"
#define ALL ((char *) (-1))
#define CIRCLE 10

enum directive {move_n,move_ne,move_e,move_se,
		move_s,move_sw,move_w,move_nw,
		button1_down,button1_up,button1_toggle,button1_click,
		button2_down,button2_up,button2_toggle,button2_click,
		button3_down,button3_up,button3_toggle,button3_click,
		mouse_activate
};

struct trans_mouse
{
	struct trans_mouse *tm_next;
	char *tm_remote;
	char *tm_button;
	enum directive tm_directive;
} *tm_first=NULL;

int always_active=1,active=0;
int acc_start=2,acc_max=20,acc_fak=2; /* defaults */
int new_always_active;
int new_acc_start,new_acc_max,new_acc_fak;

char *progname="lircmd";

int lircd,lircm;

enum {button_up,button_down}
button1=button_up,button2=button_up,button3=button_up;

struct trans_mouse *read_config(FILE *fd);

void freetm(struct trans_mouse *tm_all)
{
	struct trans_mouse *tm;

	while(tm_all!=NULL)
	{
		if(tm_all->tm_remote!=ALL && tm_all->tm_remote!=NULL)
			free(tm_all->tm_remote);
		if(tm_all->tm_button!=ALL && tm_all->tm_button!=NULL)
			free(tm_all->tm_button);
		tm=tm_all;
		tm_all=tm->tm_next;
		free(tm);
	}	
}

void sigterm(int sig)
{
	freetm(tm_first);

	shutdown(lircd,2);
	close(lircd);
	shutdown(lircm,2);
	close(lircm);
	raise(sig);
}

void sighup(int sig)
{
	FILE *fd;
	struct trans_mouse *tm_list;

	fd=fopen(CONFIGFILE,"r");
	if(fd==NULL)
	{
		syslog(LOG_WARNING,"could not open config file\n%m\n");
		return;
	}
	tm_list=read_config(fd);
	fclose(fd);
	if(tm_list==(void *) -1)
	{
		syslog(LOG_WARNING,"reading of config file failed\n");
	}
	else
	{
		freetm(tm_first);
		tm_first=tm_list;
		acc_start=new_acc_start;
		acc_max=new_acc_max;
		acc_fak=new_acc_fak;
		always_active=new_always_active;
	}
}

#ifdef DAEMONIZE
void daemonize(void)
{
	pid_t pid;
	
	if((pid=fork())<0)
	{
		fprintf(stderr,"%s: fork() failed\n",progname);
		perror(progname);
		exit(EXIT_FAILURE);
	}
	else if(pid) /* parent */
	{
		exit(EXIT_SUCCESS);
	}
	else
	{
		fclose(stdin);
		fclose(stdout);
		fclose(stderr);
		setsid();
		chdir("/");
		umask(0);
	}
}

#endif DAEMONIZE

void msend(int dx,int dy,int rep,int buttp,int buttr)
{
	static int buttons=0;
	int f=1;
	char buf[5];
	
	if(rep>=acc_start)
	{
		if(rep*acc_fak>=acc_max)
		{
			f=acc_max;
		}
		else
		{
			f=rep*acc_fak;
		}
	}
	
	buttons|=buttp;
	buttons&=~buttr;

	buf[0]=~(buttons|0x78);

	buf[1]=dx; 
	buf[2]=dy;
	buf[3]=buf[4]=0;

	while(f>0)
	{
		f--;
		write(lircm,buf,5);
	}
}

void mouse_move(int dx,int dy,int rep)
{
	msend(dx,dy,rep,0,0);
}

void mouse_button(int down,int up,int rep)
{
	if(rep==0)
	{
		msend(0,0,rep,down,up);
		if(down&0x4) button1=button_down;
		if(down&0x2) button2=button_down;
		if(down&0x1) button3=button_down;
		if(up&0x4) button1=button_up;
		if(up&0x2) button2=button_up;
		if(up&0x1) button3=button_up;
	}
}

/*
  You don't understand this funktion?
  Never mind, I don't understand it, too.
*/

void mouse_circle(int r,int dirx,int diry)
{
	int i,d,incX,incY,x,y;
	int dd[8]=
	{
		1, 0,-1,-1,-1, 0, 1, 1
	};

	for(i=0;i<8;i++)
	{
		d=1-r;
		incX=0;
		incY=2*r;
		x=0;
		y=r;
		while(x<y)
		{
			if(d>=0)
			{
				y--;
				incY-=2;
				d-=incY;
				mouse_move(dirx*dd[i],
					   diry*dd[(i+8-6)%8],0);
			}
			else
			{
				mouse_move(dirx*dd[(i+8-1)%8],
					   diry*dd[(i+8-7)%8],0);
			}
			x++;
			incX+=2;
			d+=incX+1;
			usleep(1);
		}
	}
}

void activate()
{
	active=1;
	mouse_circle(CIRCLE,1,1);
}

void deactivate()
{
	active=0;
	mouse_circle(CIRCLE,-1,1);
}


void mouse_conv(int rep,char *button,char *remote)
{
	struct trans_mouse *tm;
	int found=0;
	
	tm=tm_first;
	while(tm!=NULL)
	{
		if(tm->tm_remote!=ALL)
		{
			if(strcasecmp(remote,tm->tm_remote)!=0)
			{
				tm=tm->tm_next;
				continue;
			}
		}
		if(tm->tm_button!=ALL)
		{
			if(strcasecmp(button,tm->tm_button)!=0)
			{
				tm=tm->tm_next;
				continue;
			}
		}
		if(tm->tm_directive==mouse_activate)
		{
			if(active==0 && always_active==0)
			{
				activate();
			}
		}

		if(active || always_active)
		{
			switch(tm->tm_directive)
			{
			case move_n:
				mouse_move(0,1,rep);
				break;
			case move_ne:
				mouse_move(1,1,rep);
				break;
			case move_e:
				mouse_move(1,0,rep);
				break;
			case move_se:
				mouse_move(1,-1,rep);
				break;
			case move_s:
				mouse_move(0,-1,rep);
				break;
			case move_sw:
				mouse_move(-1,-1,rep);
				break;
			case move_w:
				mouse_move(-1,0,rep);
				break;
			case move_nw:
				mouse_move(-1,1,rep);
				break;
			case button1_down:
				mouse_button(4,0,rep);
				break;
			case button1_up:
				mouse_button(0,4,rep);
				break;
			case button1_toggle:
				if(button1==button_up)
					mouse_button(4,0,rep);
				else
					mouse_button(0,4,rep);
				break;
			case button1_click:
				mouse_button(4,0,rep);
#ifdef CLICK_DELAY
				usleep(CLICK_DELAY);
#endif
				mouse_button(0,4,rep);
				break;
			case button2_down:
				mouse_button(2,0,rep);
				break;
			case button2_up:
				mouse_button(0,2,rep);
				break;
			case button2_toggle:
				if(button2==button_up)
					mouse_button(2,0,rep);
				else
					mouse_button(0,2,rep);
				break;
			case button2_click:
				mouse_button(2,0,rep);
#ifdef CLICK_DELAY
				usleep(CLICK_DELAY);
#endif
				mouse_button(0,2,rep);
				break;
			case button3_down:
				mouse_button(1,0,rep);
				break;
			case button3_up:
				mouse_button(0,1,rep);
				break;
			case button3_toggle:
				if(button3==button_up)
					mouse_button(1,0,rep);
				else
					mouse_button(0,1,rep);
				break;
			case button3_click:
				mouse_button(1,0,rep);
#ifdef CLICK_DELAY
				usleep(CLICK_DELAY);
#endif
				mouse_button(0,1,rep);
				break;
			default:
				break;
			}
		}
		found=1;
		tm=tm->tm_next;
	}
	if(found==0)
	{
		if(active==1 && always_active==0)
		{
			deactivate();
		}
	}
}

struct trans_mouse *read_config(FILE *fd)
{
	char buffer[PACKET_SIZE];
	char *directives,*remote,*button;
	enum directive d;
	int len;
	int line;
	struct trans_mouse *tm_new,*tm_list,*tm_last=NULL;

	tm_list=NULL;
	new_acc_start=acc_start;
	new_acc_max=acc_max;
	new_acc_fak=acc_fak;
	new_always_active=1;
	line=0;
	while(fgets(buffer,PACKET_SIZE,fd)!=NULL)
	{
		line++;
		len=strlen(buffer);
		if(len==PACKET_SIZE-1 && buffer[len-1]!='\n')
		{
			syslog(LOG_ERR,"line %d too long in config file\n",
				line);
			freetm(tm_list);
			return((void *) -1);
		}
		len--;
		if(buffer[len]=='\n') buffer[len]=0;

		/* ignore comments */
		if(buffer[0]=='#') continue;

		directives=strtok(buffer,WHITE_SPACE);
		/* ignore empty lines */
		if(directives==NULL) continue;
			
		if(strcasecmp("ACCELERATOR",directives)==0)
		{
			char *number;

			number=strtok(NULL,WHITE_SPACE);
			if(number!=NULL)
				new_acc_start=atoi(number);
			number=strtok(NULL,WHITE_SPACE);
			if(number!=NULL)
				new_acc_max=atoi(number);
			number=strtok(NULL,WHITE_SPACE);
			if(number!=NULL)
				new_acc_fak=atoi(number);
			if(strtok(NULL,WHITE_SPACE)!=NULL)
			{
				syslog(LOG_WARNING,
					"invalid line %d in config file ignored\n",
				       line);
				new_acc_start=acc_start;
				new_acc_max=acc_max;
				new_acc_fak=acc_fak;
				continue;
			}
			else
			{
			}
			continue;
		}

		remote=strtok(NULL,WHITE_SPACE);
		button=strtok(NULL,WHITE_SPACE);
		if(remote==NULL || button==NULL || 
		   strtok(NULL,WHITE_SPACE)!=NULL)
		{
			syslog(LOG_WARNING,
			       "invalid line %d in config file ignored\n",
			       line);
			continue;			
		}

		if(strcasecmp("ACTIVATE",directives)==0)
		{
			d=mouse_activate;
			new_always_active=0;
		}
		else if(strcasecmp("MOVE_N",directives)==0)
			d=move_n;
		else if(strcasecmp("MOVE_NE",directives)==0)
			d=move_ne;
		else if(strcasecmp("MOVE_E",directives)==0)
			d=move_e;
		else if(strcasecmp("MOVE_SE",directives)==0)
			d=move_se;
		else if(strcasecmp("MOVE_S",directives)==0)
			d=move_s;
		else if(strcasecmp("MOVE_SW",directives)==0)
			d=move_sw;
		else if(strcasecmp("MOVE_W",directives)==0)
			d=move_w;
		else if(strcasecmp("MOVE_NW",directives)==0)
			d=move_nw;
		else if(strcasecmp("BUTTON1_DOWN",directives)==0)
			d=button1_down;
		else if(strcasecmp("BUTTON1_UP",directives)==0)
			d=button1_up;
		else if(strcasecmp("BUTTON1_TOGGLE",directives)==0)
			d=button1_toggle;
		else if(strcasecmp("BUTTON1_CLICK",directives)==0)
			d=button1_click;
		else if(strcasecmp("BUTTON2_DOWN",directives)==0)
			d=button2_down;
		else if(strcasecmp("BUTTON2_UP",directives)==0)
			d=button2_up;
		else if(strcasecmp("BUTTON2_TOGGLE",directives)==0)
			d=button2_toggle;
		else if(strcasecmp("BUTTON2_CLICK",directives)==0)
			d=button2_click;
		else if(strcasecmp("BUTTON3_DOWN",directives)==0)
			d=button3_down;
		else if(strcasecmp("BUTTON3_UP",directives)==0)
			d=button3_up;
		else if(strcasecmp("BUTTON3_TOGGLE",directives)==0)
			d=button3_toggle;
		else if(strcasecmp("BUTTON3_CLICK",directives)==0)
			d=button3_click;
		else
		{
			syslog(LOG_WARNING,
			       "unknown direcitve \"%s\" ignored\n",
			       directives);
			continue;
		}
		
		if(strcmp("*",remote)==0) remote=ALL;
		else remote=strdup(remote);
		if(strcmp("*",button)==0) button=ALL;
		else button=strdup(button);
		
		tm_new=malloc(sizeof(struct trans_mouse));
		if(remote==NULL || button==NULL || tm_new==NULL)
		{
			syslog(LOG_ERR,"out of memory\n");
			if(remote!=NULL) free(remote);
			if(button!=NULL) free(button);
			if(tm_new!=NULL) free(tm_new);
			free(tm_list);
			return((void *) -1);
		}
		tm_new->tm_next=NULL;
		tm_new->tm_remote=remote;
		tm_new->tm_button=button;
		tm_new->tm_directive=d;
		if(tm_list==NULL)
		{
			tm_list=tm_new;
			tm_last=tm_new;
		}
		else
		{
			tm_last->tm_next=tm_new;
			tm_last=tm_new;
		}
	}
	return(tm_list);
}

void loop()
{
	ssize_t len=0;
	char buffer[PACKET_SIZE+1];
	int rep;
	char button[PACKET_SIZE+1];
	char remote[PACKET_SIZE+1];
	char *end;
	int end_len=0;

	buffer[0]=0;
	while(1)
	{
		if(strchr(buffer,'\n')==NULL)
		{
			len=read(lircd,buffer+end_len,PACKET_SIZE-end_len);
		}
		if(len<=0)
		{
			raise(SIGTERM);
		}
		buffer[len+end_len]=0;
		if(sscanf(buffer,"%*llx %x %s %s\n",&rep,button,remote)==3)
		{
			end=strchr(buffer,'\n')+1;
			end_len=strlen(end);
			memmove(buffer,end,end_len+1);
			mouse_conv(rep,button,remote);
		}
	}
	
}

int main()
{
	FILE *fd;
	struct sigaction act;
	struct sockaddr_un addr;

	/* connect to lircd */

	addr.sun_family=AF_UNIX;
	strcpy(addr.sun_path,LIRCD);
	lircd=socket(AF_UNIX,SOCK_STREAM,0);
	if(lircd==-1)
	{
		fprintf(stderr,"%s: could not open socket\n",progname);
		perror(progname);
		exit(EXIT_FAILURE);
	};
	if(connect(lircd,(struct sockaddr *)&addr,sizeof(addr))==-1)
	{
		fprintf(stderr,"%s: could not connect to socket\n",progname);
		perror(progname);
		exit(EXIT_FAILURE);
	};

	/* open fifo */
	
	if(mkfifo(LIRCM,0644)==-1)
	{
		if(errno!=EEXIST)
		{
			fprintf(stderr,"%s: could not create fifo\n",progname);
			perror(progname);
			exit(EXIT_FAILURE);
		}
	}
	
	lircm=open(LIRCM,O_RDWR|O_NONBLOCK);
	if(lircm==-1)
	{
		fprintf(stderr,"%s: could not open fifo\n",progname);
		perror(progname);
		exit(EXIT_FAILURE);
	}

	/* read config file */

	fd=fopen(CONFIGFILE,"r");
	if(fd==NULL)
	{
		fprintf(stderr,"%s: could not open config file\n",progname);
		perror(progname);
		exit(EXIT_FAILURE);
	}
	tm_first=read_config(fd);
	fclose(fd);
	if(tm_first==(void *) -1)
	{
		fprintf(stderr,"%s: reading of config file failed\n",progname);
		exit(EXIT_FAILURE);
	}
	else
	{
		acc_start=new_acc_start;
		acc_max=new_acc_max;
		acc_fak=new_acc_fak;
		always_active=new_always_active;
	}

#ifdef DAEMONIZE
	daemonize();
#endif
	openlog(progname,LOG_CONS,LOG_DAEMON);
	
	signal(SIGPIPE,SIG_IGN);
	signal(SIGTERM,sigterm);
	signal(SIGINT,sigterm);

	act.sa_handler=sighup;
	sigemptyset(&act.sa_mask);
	act.sa_flags=SA_RESTART;           /* don't fiddle with EINTR */
	sigaction(SIGHUP,&act,NULL);

	loop();
	
	/* never reached */
	return(EXIT_SUCCESS);
}
