/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             RD6831
  supported devices: CD-changer CDC935
*/


struct ir_ncode philips_cd_codes[]=
{
        {"0",        0x00},
        {"1",        0x01},
        {"2",        0x02},
        {"3",        0x03},
        {"4",        0x04},
        {"5",        0x05},
        {"6",        0x06},
        {"7",        0x07},
        {"8",        0x08},
        {"9",        0x09},
        {"INDEX",    0x2b},
        {"OPEN/CLOSE",       0x2d},
        {"REPEAT",   0x1d},
        {"DISC",     0x1e},
        {"PAUSE",    0x30},
        {"RANDOM",   0x1c},
        {"TRACK_+",  0x20},
        {"TRACK_-",  0x21},
        {"PLAY",     0x35},
        {"STOP",     0x36},
	{NULL, 0}
};

struct ir_remote philips_cd_remote=
{
	"PHILIPS_CD",               /* name of remote control */
	philips_cd_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x54,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             RC5420
  supported devices: unknown TV set
*/

struct ir_ncode philips_tv_codes[]=
{
	{"0", 		0x00},
	{"1", 		0x01},
	{"2", 		0x02},
	{"3", 		0x03},
	{"4", 		0x04},
	{"5", 		0x05},
	{"6", 		0x06},
	{"7", 		0x07},
	{"8", 		0x08},
	{"9", 		0x09},
	{"POWER",	0x0c},
	{"RESET",	0x0e},
	{"MUTE",	0x0d},
	{"A/V",		0x38},
	{"-/--",	0x0a},
	{"VOL_UP",	0x10},
	{"BRT_+",	0x12},
	{"CONT_+",	0x14},
	{"VOL_DOWN",	0x11},
	{"BRT_-",	0x13},
	{"CONT_-",	0x0e},
	{"unknown1",	0x32},
	{"unknown2",	0x34},
	{"TEXT_TV",	0x3c},
	{"TIMER",	0x37},
	{"unknown3",	0x36},
	{"unknown4",	0x35},
	{NULL, 0}
};

struct ir_remote philips_tv_remote=
{
	"PHILIPS_TV",               /* name of remote control */
	philips_tv_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x40,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_tv1_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},	
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},	
	{"7",		0x07},	
	{"8",		0x08},
	{"9",		0x09},	
	{"0",		0x00},
	{"A/V",		0x38},
	{"TEXT_TV",	0x3c},	
	{"POWER",	0x0c},	
	{"-/--",	0x0a},	
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_tv1_remote=
{
	"PHILIPS_MULTI_TV1",        /* name of remote control */
	philips_multi_tv1_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x40,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_tv2_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},	
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},	
	{"7",		0x07},	
	{"8",		0x08},
	{"9",		0x09},	
	{"0",		0x00},
	{"A/V",		0x38},
	{"TEXT_TV",	0x3c},	
	{"TV",		0x3c},	
	{"POWER",	0x0c},	
	{"-/--",	0x22},	
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_tv2_remote=
{
	"PHILIPS_MULTI_TV2",        /* name of remote control */
	philips_multi_tv2_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x40,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_tv3_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},	
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},	
	{"7",		0x07},	
	{"8",		0x08},
	{"9",		0x09},	
	{"0",		0x00},
	{"A/V",		0x28},
	{"TEXT_TV",	0x3c},	
	{"TV",		0x3f},	
	{"POWER",	0x0c},	
	{"-/--",	0x0a},	
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_tv3_remote=
{
	"PHILIPS_MULTI_TV3",        /* name of remote control */
	philips_multi_tv3_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x40,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_tv4_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},	
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},	
	{"7",		0x07},	
	{"8",		0x08},
	{"9",		0x09},	
	{"0",		0x00},
	{"A/V",		0x38},
	{"TEXT_TV",	0x3c},	
	{"TV",		0x3f},	
	{"POWER",	0x0c},	
	{"-/--",	0x0a},	
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_tv4_remote=
{
	"PHILIPS_MULTI_TV4",        /* name of remote control */
	philips_multi_tv4_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x40,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             rt420
  supported devices: VCR VR522
*/

struct ir_ncode philips_vcr_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},	
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},	
	{"7",		0x07},	
	{"8",		0x08},
	{"9",		0x09},	
	{"0",		0x00},
	{"POWER",	0x0c},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x0c},
	{"REW",		0x25},
	{"FF",		0x26},
	{"PLAY",	0x35},
	{"STOP",	0x36},
	{"PAUSE",	0x29},
	{"REC",		0x37},
	{"SELECT",	0x0b},
	{"TRACKING",	0x0e},
	{"OSD",		0x0f},
	{"CLEAR",	0x31},
	{"GOTO",	0x33},
	{"TIMER",	0x1d},
	{"MODE",	0x1e},
	{"SP/LP",	0x3a},
	{"AUDIO",	0x23},
	{NULL, 0}
};

struct ir_remote philips_vcr_remote=
{
	"PHILIPS_VCR",              /* name of remote control */
	philips_vcr_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x45,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_vcr1_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},
	{"7",		0x07},
	{"8",		0x08},
	{"9",		0x09},
	{"0",		0x00},
	{"POWER",	0x0c},
	{"-/--",	0x0a},
	{"2-",		0x0b},
	{"REW",		0x32},
	{"FF",		0x34},
	{"PLAY",	0x35},
	{"REC",		0x37},
	{"STOP",	0x36},
	{"PAUSE",	0x29},
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_vcr1_remote=
{
	"PHILIPS_multi_VCR1",       /* name of remote control */
	philips_multi_vcr1_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x45,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_vcr2_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},
	{"7",		0x07},
	{"8",		0x08},
	{"9",		0x09},
	{"0",		0x00},
	{"A/V",		0x2b},
	{"POWER",	0x0c},
	{"-/--",	0x0a},
	{"2-",		0x0b},
	{"REW",		0x32},
	{"FF",		0x34},
	{"PLAY",	0x35},
	{"REC",		0x37},
	{"STOP",	0x36},
	{"PAUSE",	0x30},
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_vcr2_remote=
{
	"PHILIPS_multi_VCR2",       /* name of remote control */
	philips_multi_vcr2_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x45,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_vcr3_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},
	{"7",		0x07},
	{"8",		0x08},
	{"9",		0x09},
	{"0",		0x00},
	{"POWER",	0x0c},
	{"-/--",	0x0a},
	{"2-",		0x0b},
	{"REW",		0x2c},
	{"FF",		0x2e},
	{"PLAY",	0x35},
	{"REC",		0x37},
	{"STOP",	0x36},
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_vcr3_remote=
{
	"PHILIPS_multi_VCR3",       /* name of remote control */
	philips_multi_vcr3_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x46,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_vcr4_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},
	{"7",		0x07},
	{"8",		0x08},
	{"9",		0x09},
	{"0",		0x00},
	{"POWER",	0x0c},
	{"-/--",	0x0a},
	{"2-",		0x0b},
	{"REW",		0x2c},
	{"FF",		0x2e},
	{"PLAY",	0x35},
	{"REC",		0x37},
	{"STOP",	0x36},
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_vcr4_remote=
{
	"PHILIPS_multi_VCR4",       /* name of remote control */
	philips_multi_vcr4_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x45,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a shift encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             unknown
  supported devices: unknown
*/

struct ir_ncode philips_multi_vcr5_codes[]=
{
	{"1",		0x01},
	{"2",		0x02},
	{"3",		0x03},
	{"4",		0x04},
	{"5",		0x05},
	{"6",		0x06},
	{"7",		0x07},
	{"8",		0x08},
	{"9",		0x09},
	{"0",		0x00},
	{"A/V",		0x0b},
	{"POWER",	0x0c},	
	{"-/--",	0x0a},
	{"2-",		0x0b},
	{"REW",		0x25},
	{"FF",		0x26},
	{"PLAY",	0x35},
	{"REC",		0x37},
	{"STOP",	0x36},
	{"PAUSE",	0x29},
	{"VOL_UP",	0x10},
	{"VOL_DOWN",	0x11},
	{"CH_UP",	0x20},
	{"CH_DOWN",	0x21},				
	{"MUTE",	0x0d},
	{NULL, 0}
};

struct ir_remote philips_multi_vcr5_remote=
{
	"PHILIPS_multi_VCR5",       /* name of remote control */
	philips_multi_vcr5_codes,
	6,                          /* bits (length of code) */
	SHIFT_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	0,0,                        /* header */
	940,840,                    /* 1 */
	940,840,                    /* 0 */
	940,                        /* leading pulse */
	0,                          /* trailing pulse */
	0,0,                        /* foot */
	0,0,                        /* indicate repeating */

	7,                          /* length of pre_data */
	0x45,                       /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	90000,                      /* time between signals in usecs */
	2                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a space encoded remote control from philips

  contributed by Per-Henrik Persson (vajper@datorklubben.ml.org)

  brand:             Philips
  model:             FW11
  supported devices: stereo sytem FW11
*/

struct ir_ncode philips_stereo_codes[]=
{
	{"unknown1",0x6897},
	{"unknown2",0x04fb},
	{"unknown3",0x06f9},
	{"unknown4",0x30cf},
	{"unknown5",0xd827},
	{"unknown6",0xe817},
	{NULL, 0}
};

struct ir_remote philips_stereo_remote=
{
	"PHILIPS_STEREO",       /* name of remote control */
	philips_stereo_codes,
	16,                         /* bits (length of code) */
	SPACE_ENC,                  /* flags */
	20,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here
				    */
	/* pulse and space lengths of: */

	4575,4445,                  /* header */
	610,1620,                   /* 1 */
	610,500,                    /* 0 */
	0,                          /* leading pulse */
	610,                        /* trailing pulse */
	0,0,                        /* foot */
	610,1620,                   /* indicate repeating */

	16,                         /* length of pre_data */
	0x2525,                     /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	46000,                      /* time between signals in usecs */
	0                           /* 1..(pre_data_bits+bits+post_data_bits) */
};
