/*
  this is a space encoded remote control with special repeat code
  from Yamaha

  contributed by Pablo d'Angelo (pablo.angelo@student.uni-ulm.de)

  brand:             Yamaha
  model:             unknown
  supported devices: receiver RX-395-RDS
                     amplifier AX-570
		     cd player CDX-570
*/

struct ir_ncode yamaha_codes[]=
{
	{"CD_PLAY",       0xF708},
	{"CD_PAUSE/STOP", 0xF609},
	{"CD_SEARCH_>",   0xF30c},
	{"CD_SEARCH_<",   0xF20d},
	{"CD_TRACK_+",    0xF50a},
	{"CD_TRACK_-",    0xF40b},
	{"CD_DISK",       0xB04f},
	{"TAPE_PLAY",     0xFf00},
	{"TAPE_STOP",     0xFc03},
	{"TAPE_SEARCH_>", 0xFd02},
	{"TAPE_SEARCH_<", 0xFe01},
	{"TAPE_DIR_A",    0xF807},
	{"TAPE_DIR_B",    0xBf40},
	{"TAPE_REC/PAUSE",0xFb04},
	{"TAPE_REC/MUTE", 0xfa05},
	{"TAPE_A/B",      0xF906},
	{"AMP_AUX",       0xE817},
	{"AMP_TAPE1",     0xE718},
	{"AMP_TAPE2",     0xe619},
	{"AMP_TUNER",	  0xE916},
	{"AMP_CD",	  0xEa15},
	{"AMP_PHONO",	  0xEb14},
	{"AMP_SLEEP",	  0xA857},
	{"AMP_POWER",	  0xE01f},
	{"VOL_UP",        0xE51a},
	{"VOL_DOWN",      0xE41b},
	{"TUNER_PRESET_+",0xEf10},
	{"TUNER_PRESET_-",0xEe11},
	{"TUNER_ABCDE",   0xEd12},
	{"EQ_PRESET_SKIP",0xa45b},
	{"EQ_ON/FLAT",    0xa55a},
	{"PHONO_PLAY/CUT",0xf10e},
	{NULL, 0}
};

struct ir_remote yamaha_remote=
{
	"YAMAHA",                   /* name of remote control */
	yamaha_codes,
	16,                         /* bits (length of code) */
	SPACE_ENC|REVERSE,          /* flags */
	50,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here */

	/* pulse and space lengths of: */

	9050,4500,                  /* header */
	600,1650,                   /* 1 */
	600,550,                    /* 0 */
	0,                          /* leading pulse */
	600,                        /* trailing pulse */
	0,0,                        /* foot */
	9100,2300,                  /* indicate repeating */

	16,                         /* length of pre_data */
	0x857a,                     /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	50000,                      /* time between signals in usecs */
	0                           /* 1..(pre_data_bits+bits+post_data_bits) */
};

/*
  this is a space encoded remote control with special repeat code
  from Yamaha

  contributed by Pablo d'Angelo

  brand:             Yamaha
  model:             unknown
  supported devices: cd player CDX-570
*/

struct ir_ncode yamaha_cd_codes[]=
{
	{"CD_PLAY",       0xfd02},
	{"CD_PAUSE",      0xaa55},
	{"CD_STOP",       0xa956},
	{"CD_SEARCH_>",   0xf906},
	{"CD_SEARCH_<",   0xfa05},
	{"CD_TRACK_+",    0xf807},
	{"CD_TRACK_-",    0xfb04},
	{"CD_OPEN/CLOSE", 0xfe01},
	{"CD_DIMMER",     0xe11e},
	{"CD_TIME",       0xf50a},
	{"CD_PROG",       0xf30c},
	{"CD_CLEAR",      0xf20d},
	{"CD_TAPE",       0xa857},
	{"CD_RANDOM",     0xe41b},
	{"CD_REPEAT",     0xf708},
	{"CD_INDEX",      0xf40b},
	{"CD_VOL_UP",     0xe21d},
	{"CD_VOL_DOWN",   0xe31c},
	{"CD_0",          0xef10},
	{"CD_1",          0xee11},
	{"CD_2",          0xed12},
	{"CD_3",          0xec13},
	{"CD_4",          0xeb14},
	{"CD_5",          0xea15},
	{"CD_6",          0xe916},
	{"CD_7",          0xe817},
	{"CD_8",          0xe718},
	{"CD_9",          0xe619},
	{"CD_+10",        0xe51a},

	{NULL, 0}
};

struct ir_remote yamaha_cd_remote=
{
	"YAMAHA_CD",                /* name of remote control */
	yamaha_cd_codes,
	16,                         /* bits (length of code) */
	SPACE_ENC|REVERSE,          /* flags */
	50,                         /* eps (_relative_ tolerance) */
	0,                          /* detecing _very short_ pulses is
				       difficult with some remotes,
				       this is an _absolute_ tolerance 
				       to solve this problem
				       usually you can say 0 here */

	/* pulse and space lengths of: */

	9050,4500,                  /* header */
	600,1650,                   /* 1 */
	600,550,                    /* 0 */
	600,                        /* trailing pulse */
	0,0,                        /* foot */
	9100,2300,                  /* indicate repeating */

	16,                         /* length of pre_data */
	0x8679,                     /* data which the remote sends before
				       actual keycode */
	0,                          /* length of post_data */
	0x00,                       /* data which the remote sends after
				       actual keycode */
	0,0,                        /* signal between pre_data and keycode */
	0,0,                        /* signal between keycode and post_code */

	50000,                      /* time between signals in usecs */
	0                           /* 1..(pre_data_bits+bits+post_data_bits) */
};
